// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .controller('DBConfigMessageBoxCtrl', ['$window', '$timeout', '$q',
        '$rootScope', '$scope', '$uibModalInstance', 'slashDB', 'configId',
        'showCachePrompt', 'showInfo', 'hasShownInfo', 'lockActive',
        'sessionControlService',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance,
                  slashdb, configId, showCachePrompt, showInfo, hasShownInfo,
                  lockActive, sessionControlService) {
            var ctrl = this;

            ctrl.$onInit = () => {
                $scope.dbId = configId;
                $scope.showCachePrompt = showCachePrompt;
                $scope.showInfo = showInfo;
                $scope.hasShownInfo = hasShownInfo || !$scope.showInfo
                getReflectStatus($scope.dbId)
                    .then(_ => getDbConfig($scope.dbId))
                    .finally(() => {
                        sessionControlService.onSessionLost(() => {
                            $scope.reflectErrorMessage = sessionControlService.expiredMsg
                            $scope.connectStatus = 'Failed'
                        })

                        sessionControlService.onSessionRestore(() => {
                            $scope.reflectErrorMessage = ''
                            $scope.connectStatus = null
                            getReflectStatus($scope.dbId)
                                .then(_ => getDbConfig($scope.dbId))
                        })
                    })
            }

            function getDbConfig(dbId) {
                return slashdb.get(
                    '/dbdef/' + dbId + '.json',
                    {params: {guidata: ''}},
                    false, false)
                    .then((response) => {
                        let config = response.data;
                        $scope.editable = config.editable;
                        $scope.viewable = config.viewable;
                        $scope.executable = config.executable;
                        $scope.cacheable = config.cache_dbmodel;
                    })
                    .catch((response) => {
                        if (response.status === 403) {
                            $scope.editable = false;
                            $scope.viewable = false;
                            // If db_id is obtained, then it should be executable.
                            $scope.executable = true;
                            $scope.cacheable = false;
                        } else {
                            console.log(response);
                        }
                    })
            }

            function getReflectStatus(dbId){
                return slashdb.get('/settings/reflect-status/' + dbId + '.json', null, false, false)
                .then(function(response){
                        $scope.reflectErrorMessage = response.data.message;
                        $scope.connectStatus = response.data.model;
                        /*
                        If we want to show the refresh cache prompt,
                        change connect status so we can use a switch block to do so.
                        */
                        if ($scope.showCachePrompt && $scope.hasShownInfo) {
                            $scope.connectStatus = 'Cacheable';
                        }
                    }
                )
            }

            $scope.retry = function(refreshCache){
                $uibModalInstance.close();
                if (refreshCache) {
                    slashdb.get('/uncache-schema/'+configId+'.json', {params: {}}, false, false).finally(() => {
                        $scope.connect();
                    });
                } else {
                    $scope.connect();
                }
            };

            $scope.connect = function(){
                var goesToWizard = $scope.showInfo && ($scope.hasShownInfo || !$scope.showCachePrompt)
                if ($scope.connectStatus == 'Failed') {
                    // On failing, the wizard will reopen messagebox with the error
                    goesToWizard = true;
                }
                if (!$scope.hasShownInfo && $scope.showCachePrompt) {
                    $rootScope.$broadcast('open-messagebox', {
                        configId: configId,
                        showCachePrompt: $scope.showCachePrompt,
                        showInfo: $scope.showInfo,
                        hasShownInfo: true,
                    })
                } else if (goesToWizard) {
                    // If you are showing info and are ready to connect
                    $rootScope.$broadcast('open-db-wizard', {
                        configId: configId,
                        dbConfig: {},
                        doReflect: true,
                        showCachePrompt: $scope.showCachePrompt,
                        showInfo: $scope.showInfo,
                    });
                    lockActive(configId, true);
                } else {
                    slashdb.toggleDB(configId, '');
                    lockActive(configId, true);
                }
            }

            $scope.modify = function(){
                var eventData = {
                    configId: configId,
                    doReflect: undefined
                };
                $uibModalInstance.close();
                $rootScope.$broadcast('open-db-editor', eventData);
            };

            $scope.close = function(){
                $uibModalInstance.dismiss();
            };
        }
    ]);