// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
        .controller('DBConfigModalCtrl', ['$window', '$timeout', '$q',
            '$rootScope', '$scope', '$uibModalInstance', '$http', 'slashDB', 'configId',
            'selectizeService', 'looseAccess', 'sessionControlService',
        function ($window, $timeout, $q, $rootScope, $scope, $uibModalInstance, $http,
                  slashdb, configId, selectizeService, looseAccess, sessionControlService) {
            const ctrl = this;
            ctrl.sqlalchemyFormFields = {
                mssql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                oracle : ['db_host', 'db_port', 'user', 'password', 'sid', 'service_name',
                    'db_schema', 'db_oracle_connection_type'],
                db2: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                mysql: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                mariadb: ['db_host', 'db_port', 'user', 'password', 'db_name'],
                postgresql: ['db_host', 'db_port', 'user', 'password', 'db_name', 'db_schema'],
                sqlite: ['file_path'],
                sqlcipher: ['file_path', 'password'],
                databricks: ['db_host', 'db_port', 'token', 'db_name', 'db_http_path'],
                snowflake: ['account_id', 'user', 'password', 'db_name',
                    'sn_schema', 'db_role', 'db_warehouse'],
            };
            // to add non sqlachemy form fields create new object and put it in
            // call at the next line
            $scope.oracle_connection_types = {'SID': 'sid', 'Service Name': 'service_name'};
            $scope.formFields = Object.assign({}, ctrl.sqlalchemyFormFields);
            ctrl.mysqlLikeDb = ['mysql', 'mariadb'];
            ctrl.fileBasedDB = ['sqlite', 'sqlcipher'];
            $scope.newDbConfig = !Boolean(configId);
            $scope.configId = configId;
            $scope.deleteVisible = !$scope.newDbConfig;
            $scope.username = slashdb.settings.user;
            // store connstr when checkbox unchecked. stored until page reload
            $scope.tempConnstr = null;
            var allCharsets = [];
            var mysqlCharsets = [];

            // variables needed for test connection button
            let _testConnection = {showSpinner: false, success: '', error: '', alertClass: null};
            $scope.testConnection = _testConnection;
            $scope.alertClass = 'alert-danger';
            let _serverInfoKeys = [
                'db_name', 'db_type', 'file_path', 'db_host', 'db_port',
                'db_schema', 'password', 'user', 'connection'
            ];
            let disableTest = true;

            slashdb.settings.py_encodings.forEach(function(charset){
                allCharsets.push(charset.label);
                if (charset.mysql) mysqlCharsets.push(charset.label)
            });

            $scope.required_fields = {};

            $scope.getCharsets = function(dbType){
                if (ctrl.mysqlLikeDb.includes(dbType)) {
                    return mysqlCharsets;
                } else {
                    return allCharsets;
                }
            };

            ctrl.$onInit = function () {
                $scope.selectizeService = selectizeService;
                selectizeService.updateUserOptions(null);
                $scope.okVisible = false;
                $scope.modalAlert = '';
                $scope.savingInProgress = false;
                resetDbConfig();
                getDbConfig()
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            $scope.modalAlert = sessionControlService.expiredMsg
                        )
                        sessionControlService.onSessionRestore(() => {
                            resetErrors();
                            getDbConfig();
                        })
                    })
            };

            /** @returns {Promise} */
            var getDbConfig = function () {
                if ($scope.newDbConfig) {
                    return $q(function (resolve, reject) {
                        resolve();
                    });
                }

                return slashdb.get('/dbdef/' + $scope.configId + '.json', {
                    params: {guidata: ''}}, false, false)
                    .then(function (response) {
                        $scope.configData = response.data;
                        $scope.deleteVisible = $scope.configData.editable;
                        $scope.configData.override = Boolean($scope.configData.connection);
                        selectizeService.updateUserOptions($scope.configData);
                        $scope.disableTest = !$scope.configData.editable;
                        var dbEncoding = $scope.configData.db_encoding;
                        // preserve original encoding in dbconfig
                        if (!allCharsets.includes(dbEncoding)) allCharsets.push(dbEncoding);
                        if (ctrl.mysqlLikeDb.includes($scope.configData.db_type)) {
                            if (!mysqlCharsets.includes(dbEncoding)) mysqlCharsets.push(dbEncoding);
                        }
                        if (!$scope.configData.db_oracle_connection_type) {
                            $scope.configData.db_oracle_connection_type = 'sid'
                        }
                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch((response) => {
                        $scope.modalAlert = response.headers('Warning');
                    })
            };

            var resetDbConfig = function () {
                // stores database definition model, some initial values
                $scope.configData = {
                    db_type: 'sqlite',
                    autoload: false,
                    autoconnect: true,
                    cache_dbmodel: false,
                    editable: true,
                    autoload_user: {dbuser: null, dbpass: null},
                };
            };

            $scope.fixServerInfo = function () {
                let neededServerInfo = ctrl.sqlalchemyFormFields[$scope.configData['db_type']];
                neededServerInfo.push('connection')
                let serverInfo = angular.copy($scope.configData);
                for (var i=0; i < neededServerInfo.length; i++){
                    var key = neededServerInfo[i];
                    if (serverInfo[key] === undefined){
                        serverInfo[key] = null;
                    }
                }
                return serverInfo;
            }

            /** @returns {boolean} */
            $scope.requiresCredentials = function () {
                return $scope.configData.use_user_db_credentials_for_mapping &&
                    $scope.configData.db_type !== 'sqlite' &&
                    $scope.configData.autoload &&
                    $scope.isSQLAlchemy();
            };

            $scope.isSQLAlchemy = function () {
                return ctrl.sqlalchemyFormFields.hasOwnProperty($scope.configData.db_type);
            };

            $scope.delete = function () {
                slashdb.deleteDBDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            $scope.modalAlert = sessionControlService.expiredMsg
                        )

                        sessionControlService.onSessionRestore(() => {
                            // refresh current user
                            slashdb.setupSettings()
                            resetErrors();
                        })
                    });
            };

            $scope.translateServerInfo = function(serverInfo) {
                switch (serverInfo.db_type) {
                    case 'sqlite':
                    case 'sqlcipher':
                        serverInfo['file_path'] = serverInfo.db_name;
                        break;
                }
                serverInfo['user'] = serverInfo.user ? serverInfo.user : serverInfo.autoload_user.dbuser;
                serverInfo['password'] = serverInfo.password ? serverInfo.password : serverInfo.autoload_user.dbpass;
                return serverInfo
            }

            $scope.testDbConnection = function() {
                let serverInfo = $scope.fixServerInfo();
                serverInfo = $scope.translateServerInfo(serverInfo);

                $scope.testConnection = angular.copy(_testConnection);
                $scope.disableTest = true;
                $scope.validateServerForm = true;
                $scope.testConnection.error = "";
                $scope.testConnection.showSpinner = true;
                return $http.post('/settings/check-database-connection.json', serverInfo)
                    .then(function successCallback(response){
                        $scope.testConnection.alertClass = 'alert-success';
                        if (response.headers('X-Error-Level') === '30') {
                            $scope.testConnection.alertClass = 'alert-warning'
                        }
                        if (response.headers('warning')) {
                            $scope.testConnection.error = `Connection Successful. ${response.headers('warning')}`;
                        } else {
                            $scope.testConnection.error = response.data;
                        }
                    })
                    .catch(function errorCallback(response){
                        if (response.data.description) {
                            $scope.testConnection.error = response.data.description;
                        } else {
                            $scope.testConnection.error = 'Unknown error';
                        }

                        $scope.testConnection.alertClass = 'alert-danger';
                        return $q.reject();
                    })
                    .finally(function(){
                        $scope.testConnection.showSpinner = false;
                        $scope.disableTest = !$scope.configData.editable;
                    });
            };

            /**
             * @param {string} dbId
             * @param {boolean} useForMapping
             */
            var sendAddBtnMsg = function (dbId, useForMapping) {
                var addBtnMsg = useForMapping ? '' :
                    'Click to associate this user with database "' + dbId + '" login / password.';
                $rootScope.$broadcast('open-user-settings-modal', addBtnMsg);
            };

            $scope.fieldVisible = function(fieldName) {
                return $scope.formFields[$scope.configData.db_type].includes(fieldName);
            };

            var setInvalidFields = function(errors) {
            var invalid = [];
                // get list of invalid fields
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'INVALID') {
                            invalid.push(err['field']);
                        }
                    }
                );

                // mark fields on form as invalid
                invalid.forEach(function (value) {
                    // check that field exist on form
                    if (!$scope.configForm.hasOwnProperty(value)) {
                        console.log('Warning - bad property ' + value);
                        return
                    }
                    // set invalid
                    $scope.configForm[value].$setValidity('server', false)
                });
                return invalid
            };

            var setRequiredFields = function (errors) {
                const alwaysRequired = ['db_id', 'db_type', 'db_encoding'];

                let required = {};
                errors.forEach(
                    function (err, _, __) {
                        if (err['type'] === 'MISSING') {
                            required[err['field']] = true;
                        }
                    }
                );

                alwaysRequired.forEach(function (value) {
                    required[value] = true;
                });

                $scope.required_fields = required;
            };

            var getUserFriendlyErrorMessages = function (errors) {
                return 'Errors found in incoming database definition: ' +
                    errors.map(
                    function (err, _, __) {
                        switch (err['type']) {
                            case 'MISSING':
                                return err['description']
                            case 'INVALID':
                                switch (err['field']) {
                                    case 'db_encoding':
                                        return 'Database charset is obligatory'
                                    case 'execute':
                                        return '"Connect / Disconnect DB": ' + err['description']
                                    case 'read':
                                        return '"View DB Config" ' + err['description']
                                    case 'write':
                                        return '"Modify DB Config": ' + err['description']
                                    default:
                                        return err['description']
                                }
                            case 'EXTRA':
                                return err['description'] + " - try to reload the page"
                            default:
                                return 'Unexpected error - try to reload the page'
                        }
                    }
                ).join(', ') + '.'
            };

            var resetErrors = function() {
                $scope.modalAlert = '';
                $scope.alertClass = 'alert-danger'
                $scope.required_fields = {};
                for (var key in $scope.configForm) {
                    var value = $scope.configForm[key];
                    if (typeof value === 'object'
                        && value.hasOwnProperty('$modelValue')) {
                        value.$setValidity('server', true)
                    }
                }
            };

            $scope.getDropdownDbTypes = () => {
                // return only licensed db types and the current one
                const sdbDbTypes = slashdb.settings.dbtypes;
                let dropdownItems = {};
                for (const db_type in sdbDbTypes) {
                    if (sdbDbTypes[db_type]['licensed']
                        || db_type === $scope.configData.db_type) {
                        dropdownItems[db_type] = sdbDbTypes[db_type]['userfriendly_name'];
                    }
                }
                return dropdownItems;
            };

            $scope.updateConnstr = function() {
                if ($scope.configData.override) {
                    $scope.configData.connection = $scope.tempConnstr;
                } else {
                    $scope.tempConnstr = $scope.configData.connection;
                    $scope.configData.connection = null;
                }
            };

            $scope.resetError = function (field) {
                $scope.configForm[field].$setValidity('server', true)
            };

            var setErrors = function(errors, description) {
                $scope.alertClass = 'alert-danger';
                if (errors !== undefined && errors.length > 0){
                    $scope.modalAlert = getUserFriendlyErrorMessages(errors);
                    setInvalidFields(errors);
                    setRequiredFields(errors);
                } else {
                    $scope.modalAlert = description;
                }
            };

            var stripReadOnlyItems = function(rawData) {
                var payload = Object.assign({}, rawData);
                delete payload.editable;
                delete payload.viewable;
                delete payload.executable;
                delete payload.cacheable;
                delete payload.cache_exists;
                delete payload.connect_status;
                delete payload.override;
                delete payload.file_path;

                return payload
            };

            let nullifyNonSqliteFields = (payload) => {
                payload.db_port = null;
                payload.db_host = null;
                payload.db_schema = null;

                return payload
            };

            $scope.save = function () {
                $scope.savingInProgress = true;
                let payload = stripReadOnlyItems($scope.configData);

                if (ctrl.fileBasedDB.includes($scope.configData.db_type)) {
                    // remove non relevant fields
                    payload = nullifyNonSqliteFields(payload);

                    payload.db_http_path = undefined;

                    payload.db_role = undefined;
                    payload.db_warehouse = undefined;
                    payload.sn_schema = undefined;
                    payload.db_oracle_connection_type = undefined;
                    // rest known db types
                } else if  ($scope.configData.db_type === 'snowflake') {
                    payload.db_schema = null;
                    // only used in databricks
                    payload.db_http_path = undefined;
                    payload.db_port = null;
                } else if (Object.keys(ctrl.sqlalchemyFormFields).includes($scope.configData.db_type)) {
                    // convert port to int
                    if (!$scope.configForm.db_port.$error.pattern) {
                        payload.db_port = parseInt(payload.db_port, 10);
                    }

                    if (ctrl.mysqlLikeDb.includes($scope.configData.db_type)) {
                        // mysql do not have schema
                        payload.db_schema = null;
                    }
                    if ($scope.configData.db_type === 'databricks') {
                        // databricks do not have schema
                        payload.db_schema = null;
                    } else {
                        // only used in databricks
                        payload.db_http_path = undefined;

                        // only used in snowflake
                        payload.db_role = undefined;
                        payload.db_warehouse = undefined;
                        payload.sn_schema = undefined;
                    }
                    if ($scope.configData.db_type !== 'oracle') {
                        payload.db_oracle_connection_type = undefined;
                    }
                } else {
                    console.log('Error, unknown db type')
                }

                resetErrors();

                processSaveRequest(payload)
                    .catch(function (response) {
                        setErrors(response.data.errors, response.data.description);
                    })
                    .finally(() => {
                        $scope.savingInProgress = false
                        sessionControlService.onSessionLost(() => {
                                $scope.modalAlert = sessionControlService.expiredMsg;
                                $scope.alertClass = 'alert-danger';
                            }
                        )

                        sessionControlService.onSessionRestore(() => {
                            // refresh current user
                            slashdb.setupSettings()
                            resetErrors();
                        })
                    });
            };

            let processSaveRequest = (payload) => {
                if ($scope.newDbConfig) {
                    return slashdb.createDBDef(payload)
                        .then(function (response) {
                            /** @type {boolean} */
                            var useForMapping = $scope.configData.use_user_db_credentials_for_mapping || false;

                            blinkOK();
                            $scope.newDbConfig = false;
                            $scope.configId = $scope.configData.db_id;
                            getDbConfig().then(function () {
                                sendAddBtnMsg($scope.configId, useForMapping);
                            });
                            setRequiredFields([]);
                        })
                } else {
                    return slashdb.updateDBDef($scope.configData.db_id, payload)
                        .then(function (response) {
                            blinkOK();
                            getDbConfig();
                            setRequiredFields([]);
                        })
                }
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            $scope.hasAccess = looseAccess.hasAccess;

            var blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };
        }
    ])
    .directive('emptyToNull', function () {
    return {
        restrict: 'A',
        require: 'ngModel',
        link: function (scope, elem, attrs, ctrl) {
            ctrl.$parsers.push(function(viewValue) {
                if(viewValue === "") {
                    return null;
                }
                return viewValue;
            });
        }
    };
    });
