// @ts-check
'use strict';

// @ts-ignore
angular.module('sdbgui')
    .component('dbconfigs', {
        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs.template.html',
        controller: ['$uibModal', '$scope', '$interval', 'sortingService', 'slashDB', 'sessionControlService',
            function DbConfigController($uibModal, $scope, $interval, sortingService, slashdb, sessionControlService) {
                const ctrl = this;

                ctrl.$onInit = function() {
                    ctrl.sort = sortingService;
                    ctrl.connectStatus = {};
                    ctrl.addConfigVisible = false;
                    ctrl.dbconfigs = [];
                    ctrl.connectStatusesInterval = null;
                    ctrl.settings = slashdb.settings;
                    ctrl.lockedActives = {};
                    $scope.searchable_fields = ['db_id', 'db_type', 'desc'];

                    window.addEventListener('focus', ctrl.onFocus);
                    window.addEventListener('blur', ctrl.onBlur);
                    ctrl.updateConfigList();
                    updateConnectStatuses();

                    $scope.$on('$destroy', function () {
                        $interval.cancel(ctrl.connectStatusesInterval);
                    });

                    $scope.$on('open-messagebox', function(event, eventData){
                        ctrl.openMessageBox(eventData);
                    });

                    $scope.$on('open-db-wizard', function(event, eventData){
                        ctrl.openWizard(eventData);
                    });

                    $scope.$on('open-db-editor', function(event, eventData){
                        ctrl.openModal(eventData.configId);
                    });

                    // open wizard on ?new in qs
                    if (window.location.search.includes('new')) {
                        ctrl.openWizard(undefined);
                    }
                }

                function getConnectStatuses(){
                    if (ctrl.dbconfigs.len == 0){
                        return null
                    }

                    slashdb.get('/settings/reflect-status.json', {}, false, false)
                        .then(function(response){
                            var connectStatuses = response.data;
                            angular.forEach(ctrl.dbconfigs, function (dbconf) {
                                dbconf.connect_status = connectStatuses[dbconf.db_id].model;
                                ctrl.lockedActives[dbconf.db_id] = false;
                            });
                        })
                        .catch(function(){
                            cancelConnectStatusInterval();
                        })
                }

                function updateConnectStatuses(){
                    if (ctrl.connectStatusesInterval === null) {
                        ctrl.connectStatusesInterval = $interval(getConnectStatuses, 1000, 0);
                    }
                }

                function cancelConnectStatusInterval(){
                    $interval.cancel(ctrl.connectStatusesInterval);
                    ctrl.connectStatusesInterval = null;
                }

                ctrl.onFocus = function () {
                    getConnectStatuses();
                    // Starts an interval for connect status when the page is focused
                    updateConnectStatuses();
                }
                ctrl.onBlur = function () {
                    // Stops the interval for connect status when the page loses focus
                    cancelConnectStatusInterval();
                }

                ctrl.updateConfigList = function () {
                    slashdb.get('/dbdef.json?guidata', {}, false, false)
                        .then(function (response) {
                            var dbconfigs = [];
                            ctrl.addConfigVisible = response.data.create_database;
                            // @ts-ignore
                            angular.forEach(response.data.dbdefs, function (dbconf) {
                                var original_dbconfig = ctrl.dbconfigs.find(function(element) {
                                    return element.db_id == dbconf.db_id
                                });
                                if (original_dbconfig) {
                                    dbconf['connect_status'] = original_dbconfig.connect_status;
                                }
                                dbconfigs.push(dbconf);
                            });
                            ctrl.listAlert = '';
                            ctrl.dbconfigs = dbconfigs;
                        })
                        .catch(function (response) {
                            if ("create_database" in response.data) {
                                ctrl.addConfigVisible = response.data.create_database;
                            }
                            ctrl.listAlert = response.headers('Warning');
                            ctrl.dbconfigs = [];
                        })
                        .finally(() => {
                            sessionControlService.onSessionLost(() =>
                                ctrl.listAlert = sessionControlService.expiredMsg
                            )
                            sessionControlService.onSessionRestore(()=>{
                                ctrl.updateConfigList()
                            })
                        });
                };

                ctrl.Statuses = {
                    Failed: 'Failed',
                    Connected: 'Connected',
                    Disconnected: 'Disconnected',
                    all: []
                };
                ctrl.Statuses.all = [ctrl.Statuses.Failed, ctrl.Statuses.Connected, ctrl.Statuses.Disconnected];

                ctrl.getStatusStyle = function (status) {
                    if (status == ctrl.Statuses.Connected) return 'bg-success';
                    if (status == ctrl.Statuses.Failed) return 'bg-danger';
                };

                ctrl.isLoading = function (dbConfig) {
                    return !ctrl.Statuses.all.includes(dbConfig.connect_status);
                };

                ctrl.isLocked = function (dbConfig) {
                    return ctrl.lockedActives[dbConfig.db_id]
                }

                ctrl.redirectToggleDBState = function (dbConfig) {
                    var showPrompt = dbConfig.cache_dbmodel && dbConfig.cache_exists;

                    if (showPrompt && [ctrl.Statuses.Disconnected, ctrl.Statuses.Failed].includes(dbConfig.connect_status)) {
                        // Opens the 'Would you like to load from cache? Yes, No' pop up
                        ctrl.openMessageBox({
                            configId: dbConfig.db_id,
                            showCachePrompt: true,
                            showInfo: false,
                            hasShownInfo: true,
                        });
                    } else {
                        ctrl.toggleDBState(dbConfig);
                    }
                }

                ctrl.openInfo = function (dbConfig) {
                    var notConnecting = !(dbConfig.connect_status in [ctrl.Statuses.Disconnected, 'Cacheable']);
                    var showPrompt = dbConfig.cache_dbmodel && notConnecting && dbConfig.cache_exists;

                    if (dbConfig.connect_status == ctrl.Statuses.Disconnected ||
                     dbConfig.connect_status == ctrl.Statuses.Failed) {
                        // Opens the 'connect, edit, close' pop up
                        ctrl.openMessageBox({
                            configId: dbConfig.db_id,
                            showCachePrompt: showPrompt,
                            showInfo: true,
                            hasShownInfo: false,
                        });
                    } else {
                        ctrl.openStatus(dbConfig);
                    }
                }

                ctrl.toggleDBState = function (dbConfig) {
                    var connectStatus = dbConfig.connect_status;
                    dbConfig.connect_status = '';
                    slashdb.toggleDB(dbConfig.db_id, connectStatus).finally(
                        function () {
                          ctrl.updateConfigList();
                    })
                };

                ctrl.openModal = function (configId) {
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-modal.template.html',
                        size: 'lg',
                        backdrop: 'static',
                        controller: 'DBConfigModalCtrl',
                        resolve: {
                            configId: function () {
                                return configId;
                            }
                        }
                    });
                    modalInstance.result
                        .catch(() => {}) // silence warning
                        .finally(function () {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };


                ctrl.openWizard = function(wizardOptions){
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-wizard.template.html',
                        size: 'lg',
                        backdrop: 'static',
                        controller: 'DBConfigWizardCtrl',
                        controllerAs: 'ctrl',
                        resolve: {
                            wizardOptions: function () {
                                return wizardOptions;
                            },
                            lockActive: function () {
                                // Lets us lock the power button without needing to send a request
                                return (db_id, value) => {
                                    ctrl.lockedActives[db_id] = value;
                                }
                            }
                        }
                    });
                    modalInstance.result
                        .catch(() => {}) // silence warning
                        .finally(function () {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };

                ctrl.openMessageBox = function(messageBoxOptions){
                    var modalInstance = $uibModal.open({
                        templateUrl: '/static/components/sdbgui-dbconfigs/sdbgui-dbconfigs-messagebox.template.html',
                        size: 'lg',
                        windowTopClass: 'center-modal',
                        controller: 'DBConfigMessageBoxCtrl',
                        controllerAs: 'ctrl',
                        resolve: {
                            configId: function () {
                                return messageBoxOptions.configId;
                            },
                            showCachePrompt: function () {
                                return messageBoxOptions.showCachePrompt;
                            },
                            showInfo: function () {
                                return messageBoxOptions.showInfo;
                            },
                            hasShownInfo: function () {
                                return messageBoxOptions.hasShownInfo;
                            },
                            lockActive: function () {
                                // Lets us lock the power button without needing to send a request
                                return (db_id, value) => {
                                    ctrl.lockedActives[db_id] = value;
                                }
                            },
                        }
                    });
                    modalInstance.result
                        .finally(function (reason) {
                            // refresh config list after closing modal
                            ctrl.updateConfigList();
                        });
                };

                ctrl.openStatus = function(dbConfig){
                    // Opens the table of reflect statuses (page 3 of the wizard)
                    ctrl.openWizard({
                        configId: dbConfig.db_id,
                        dbConfig: dbConfig,
                        doReflect: false
                    });
                };
            }
        ]
    });
