// Generated by CoffeeScript 2.7.0
(function() {
  var QuerystudioController,
    hasProp = {}.hasOwnProperty,
    indexOf = [].indexOf;

  QuerystudioController = (function() {
    var methodsMapping, methodsMappingKeys, methodsMappingKeysL;

    class QuerystudioController {
      constructor($window, $location, $timeout, $rootScope, $scope, slashdb, QES, selectizeService, sessionControlService) {
        this.$window = $window;
        this.$location = $location;
        this.$timeout = $timeout;
        this.$rootScope = $rootScope;
        this.$scope = $scope;
        this.slashdb = slashdb;
        this.QES = QES;
        this.selectizeService = selectizeService;
        this.sessionControlService = sessionControlService;
        this.tabs = {
          side: {
            config: true,
            schema: false
          },
          top: {
            sql: true,
            diagram: false
          },
          bottom: {
            execute: true,
            results: false
          }
        };
        this.required_fields = {};
        this.executeOnly = null;
        this.codemirrorInstance = null;
        this.lastDbSchemaId = null;
        this.execUrls = {
          html: '',
          json: '',
          csv: '',
          xml: '',
          xsd: ''
        };
        // reinit forms on page restore from cache in FF
        this.$window.onpageshow = (event) => {
          if (event.persisted) {
            return this.$onInit();
          }
        };
        this.fieldMappingQueryDef = {
          'query_id': 'Query ID',
          'database': 'Database',
          'read': 'View',
          'write': 'Edit',
          'execute': 'Execute',
          'http_methods': 'HTTP Methods',
          'sqlstr': 'SQL Statement',
          'desc': 'Description'
        };
        this.savingInProgress = false;
        this.executionInProgress = false;
        this.nullPlaceHolder = "<SDB_NULL>";
        this.$window.addEventListener('beforeunload', (event) => {
          if (this.executionInProgress) {
            return event.preventDefault();
          }
        });
      }

      // Custom message is not supported on modern browsers.
      $onInit() {
        this.slashdb.setupSettings().then(() => {
          this.initCodemirror();
          this.initConfigForm();
          this.initExecuteForm();
          this.initResults();
          this.initSchema();
          return this.updateConfigForm().then(() => {
            var executeNow, res;
            this.updateCodemirror();
            executeNow = this.restoreExecuteForm();
            if (executeNow) {
              this.updateExecuteForm(this.configData);
              res = this.executeQuery();
              if (!res) {
                return this.switchTab('bottom', 'execute');
              }
            }
          });
        }).finally(() => {
          // fix for renaming query when public in edit #1080
          if (this.slashdb.settings.user === 'public') {
            this.sessionControlService.onSessionLost(() => {
              return setErrors(null, this.sessionControlService.expiredMsg);
            });
            return this.sessionControlService.onSessionRestore(() => {
              return this.$onInit();
            });
          }
        });
        return null;
      }

      initSchema() {
        return this.$scope.$watch(() => {
          return this.configData.database;
        }, (newValue, oldValue) => {
          if (newValue && newValue !== oldValue) {
            return this.updateHints(newValue);
          }
        });
      }

      updateHints(dbId) {
        var url;
        if (!dbId) {
          return;
        }
        url = `/db/${dbId}.json?tables`;
        return this.slashdb.get(url).then((response) => {
          var column, j, l, len, len1, ref, schemaHints, schemaTables, table;
          schemaTables = response.data;
          schemaHints = new Set([]);
          for (j = 0, len = schemaTables.length; j < len; j++) {
            table = schemaTables[j];
            schemaHints.add(table.name);
            ref = table.columns;
            for (l = 0, len1 = ref.length; l < len1; l++) {
              column = ref[l];
              schemaHints.add(column.name);
            }
          }
          return this.schemaHints = Array.from(schemaHints);
        });
      }

      initConfigForm() {
        this.$scope.databases = [];
        this.slashdb.get('/db.json', {
          params: {
            guidata: ''
          }
        }, false, false).then((response) => {
          var dbId, dbInfo, ref;
          ref = response.data;
          for (dbId in ref) {
            dbInfo = ref[dbId];
            dbInfo.db_id = dbId;
            this.$scope.databases.push(dbInfo);
          }
        });
        // stores database definition model, some initial values
        this.configData = {
          editable: true
        };
        this.updateSuggestedMethods();
        this.$scope.newConfig = !Boolean(this.queryId);
        this.$scope.configId = this.queryId;
        this.$scope.okVisible = false;
        this.$scope.errorMessage = '';
        return this.$scope.databases = [];
      }

      initExecuteForm() {
        this.httpMethods = ['GET', 'POST', 'PUT', 'DELETE'];
        this.extensions = ['html', 'json', 'csv', 'xml', 'xsd'];
        this.defaultLimit = parseInt(this.slashdb.settings.html_limit) || 29;
        this.ready = false;
        this.limit = this.defaultLimit;
        this.offset = 0;
        this.count = false;
        this.format = 'json';
        this.distinct = false;
        this.transpose = false;
        this.stream = false;
        this.nullStr = false;
        this.sortMap = new Map();
        this.execUrl = '';
        this.execPath = '';
        this.queryExecuted = false;
        this.selectedHTTPMethod = null;
        this.queryParameters = {};
        this.parameterNullState = {};
        this.$scope.methodAvailable = (m) => {
          var http_methods;
          http_methods = this.configData.http_methods;
          if (http_methods) {
            return http_methods[m];
          }
        };
        return this.$scope.hasParameters = () => {
          return Object.keys(this.queryParameters).length > 0;
        };
      }

      initResults() {
        this.queryOutput = null;
        return this.$rootScope.$on('queryExecuted', (event, args) => {
          var httpResponse, noData;
          this.executionInProgress = false;
          this.queryOutput = this.QES.queryOutput;
          httpResponse = args.httpResponse;
          this.resultsResponseCode = httpResponse.status;
          this.resultsResponseMessage = `${httpResponse.status} ${httpResponse.statusText}`;
          this.resultsResponseWarning = httpResponse.headers('Warning');
          this.resultsCount = httpResponse.headers(`${this.slashdb.settings.branding_name}-all-record-count`);
          this.resultsCountFailed = this.resultsResponseWarning === 'Failed to get count.';
          noData = ((httpResponse.data != null) && httpResponse.data.length === 0) || (httpResponse.data == null);
          if (httpResponse.status < 400 && noData) {
            this.resultsResponseNoMoreData = true;
          } else {
            this.resultsResponseNoMoreData = false;
          }
          if (httpResponse.status < 400) {
            this.transposeResultTable = this.transpose;
            if (this.transpose) {
              this.queryOutput.data = this.queryOutput.data[0];
            }
          }
          return this.switchTab('bottom', 'results');
        });
      }

      initCodemirror() {
        var handleLoadedEditor, produceHints;
        handleLoadedEditor = (_editor) => {
          this.codemirrorInstance = _editor;
          return _editor.on('change', () => {
            return this.updateSuggestedMethods();
          });
        };
        produceHints = (mc, opts) => {
          var dbschemaHints, hint, hints, j, len, ref;
          hints = CodeMirror.hint.sql(mc, opts);
          dbschemaHints = CodeMirror.hint.dbschema(mc, opts, this.schemaHints);
          if (dbschemaHints) {
            ref = dbschemaHints.list;
            for (j = 0, len = ref.length; j < len; j++) {
              hint = ref[j];
              hints.list.push(hint);
            }
          }
          return hints;
        };
        // Has to change in value to apply changes in CodeMirror settings
        this.$scope.refreshCodemirror = true;
        return this.$scope.editorOptions = {
          onLoad: handleLoadedEditor,
          lineNumbers: true,
          readOnly: false,
          mode: 'text/x-sql',
          hintOptions: {
            hint: produceHints
          },
          extraKeys: {
            "Ctrl-Space": "autocomplete"
          }
        };
      }

      updateCodemirror() {
        var cmMode, currDbType, ref, ref1, ref2;
        currDbType = (ref = this.$scope.databases) != null ? (ref1 = ref[this.configData.database]) != null ? ref1.db_type : void 0 : void 0;
        cmMode = (function() {
          switch (currDbType) {
            case 'sqlite':
              return "text/x-sqlite";
            case 'mssql':
              return 'text/x-mssql';
            case 'oracle':
              return 'text/x-plsql';
            case 'mysql':
              return 'text/x-mysql';
            case 'postgresql':
              return 'text/x-gpsql';
            case 'db2':
              return 'text/x-sql';
            default:
              return 'text/x-sql';
          }
        })();
        this.$scope.editorOptions.mode = cmMode;
        this.$scope.editorOptions.readOnly = !((ref2 = this.configData) != null ? ref2.editable : void 0) ? 'nocursor' : false;
        return this.$scope.refreshCodemirror = !this.$scope.refreshCodemirror;
      }

      switchTab(section, tabName) {
        var j, len, ref, results, tab;
        ref = Object.keys(this.tabs[section]);
        results = [];
        for (j = 0, len = ref.length; j < len; j++) {
          tab = ref[j];
          results.push(this.tabs[section][tab] = tab === tabName);
        }
        return results;
      }

      updateConfigForm() {
        var promise;
        if (this.$scope.newConfig) {
          this.executeOnly = false;
          this.configData.executable = true;
          this.updateCodemirror();
          return this.selectizeService.updateUserOptions(null);
        } else {
          this.$scope.errorMessage = '';
          promise = this.slashdb.get('/querydef/' + this.$scope.configId + '.json', {
            params: {
              guidata: ''
            }
          }, false, false).then((response) => {
            this.configData = angular.copy(response.data);
            this.executeOnly = this.configData.executable && !(this.configData.editable || this.configData.viewable);
            if (this.configData.editable || this.configData.viewable) {
              this.selectizeService.updateUserOptions(this.configData);
            }
            this.updateSuggestedMethods();
            this.updateCodemirror();
            this.updateExecuteForm(this.configData);
          }).catch((response) => {
            return this.$scope.errorMessage = response.headers('Warning');
          });
          return promise;
        }
      }

      clean(andDisable) {
        this.configData = {
          editable: !andDisable
        };
      }

      delete() {
        this.$scope.errorMessage = '';
        this.slashdb.deleteQueryDef(this.$scope.configId).then((response) => {
          this.slashdb.setupSettings();
          this.$window.location.href = '/querydef';
        }).catch((response) => {
          this.$scope.errorMessage = response.headers('Warning');
        }).finally(() => {
          this.sessionControlService.onSessionLost(() => {
            return setErrors(null, this.sessionControlService.expiredMsg);
          });
          return this.sessionControlService.onSessionRestore(() => {
            // update current user after login
            this.slashdb.setupSettings();
            return this.resetErrors();
          });
        });
      }

      saveExecuteForm(execute) {
        var e, formParams;
        formParams = {
          format: this.format,
          limit: this.limit,
          offset: this.offset,
          count: this.count,
          distinct: this.distinct,
          transpose: this.transpose,
          stream: this.stream,
          selectedHTTPMethod: this.selectedHTTPMethod,
          queryParameters: this.queryParameters,
          queryId: this.configData.query_id,
          executeNow: execute
        };
        try {
          return localStorage.setItem('qsExecuteQuery', JSON.stringify(formParams));
        } catch (error1) {
          e = error1;
          if (e === QUOTA_EXCEEDED_ERR) {
            return alert('local storage limit exceeded');
          }
        }
      }

      restoreExecuteForm() {
        var formParams;
        formParams = JSON.parse(localStorage.getItem('qsExecuteQuery'));
        if (formParams == null) {
          return false;
        }
        localStorage.removeItem('qsExecuteQuery');
        if (formParams.queryId !== this.queryId) {
          return false;
        }
        this.limit = formParams.limit;
        this.offset = formParams.offset;
        this.count = formParams.count;
        this.distinct = formParams.distinct;
        this.transpose = formParams.transpose;
        this.stream = formParams.stream;
        this.format = formParams.format;
        this.selectedHTTPMethod = formParams.selectedHTTPMethod;
        this.queryParameters = formParams.queryParameters;
        return formParams.executeNow;
      }

      setRequiredFields(errors) {
        var err, j, len, required;
        required = {};
        for (j = 0, len = errors.length; j < len; j++) {
          err = errors[j];
          if (err.type === 'MISSING') {
            required[err.field] = true;
          }
          // treat empty sqlstr as required
          if (err.field === 'sqlstr' && err.type === 'INVALID') {
            required[err.field] = true;
          }
          // treat empty query id as required
          if (err.field === 'query_id' && err.type === 'INVALID' && !this.configData.query_id) {
            required[err.field] = true;
          }
          // http methods can only be invalid when missing
          if (err.field === 'http_methods') {
            required[err.field] = true;
          }
        }
        ['query_id', 'database', 'http_methods', 'sqlstr'].forEach(function(value, _, __) {
          return required[value] = true;
        });
        return this.required_fields = required;
      }

      setInvalidFields(errors) {
        var err, j, len, results;
        results = [];
        for (j = 0, len = errors.length; j < len; j++) {
          err = errors[j];
          if (err.type === 'INVALID' && err.field in this.configForm) {
            // treat empty query id as required, but not invalid
            if (err.field === 'query_id' && !this.configData.query_id) {
              continue;
            }
            // treat empty sqlstr as required, but not invalid
            if (err.field === 'sqlstr' && err['description'].includes('is too short')) {
              continue;
            }
            results.push(this.configForm[err.field].$setValidity('valid', false));
          } else {
            results.push(void 0);
          }
        }
        return results;
      }

      getUserFriendlyErrorMessages(errors) {
        return 'Errors found in incoming query definition: ' + errors.map((err) => {
          var isEmptyHttpMethods, ref;
          if (err['type'] === 'MISSING') {
            if (err['field'] === 'http_methods') {
              return "At least one HTTP Method must be selected";
            }
            return `${this.fieldMappingQueryDef[err['field']]} is obligatory`;
          }
          if (err['type'] === 'EXTRA') {
            return `${err['description']} - try to reload page`;
          }
          if (err['field'] === 'sqlstr' && err['description'].includes('is too short')) {
            return `${this.fieldMappingQueryDef[err['field']]} is required`;
          }
          if (err['field'] === 'query_id' && !this.configData.query_id) {
            return `${this.fieldMappingQueryDef[err['field']]} is required`;
          }
          if (err['field'] === 'query_id' && err['description'].includes('already exists')) {
            return err['description'];
          }
          if ((this.configData.http_methods == null) || Object.keys(this.configData.http_methods).length === 0) {
            isEmptyHttpMethods = true;
          } else {
            isEmptyHttpMethods = false;
          }
          if (err['field'] === 'http_methods' && isEmptyHttpMethods) {
            return "At least one HTTP Method must be selected";
          }
          if (((ref = err['field']) === 'execute' || ref === 'read' || ref === 'write') && err['description'].includes('is not valid under')) {
            return `'${this.fieldMappingQueryDef[err['field']]}': ${err['description']}`;
          }
          if (err['description'].includes('is too long')) {
            return `${this.fieldMappingQueryDef[err['field']]} is too long`;
          }
          return `${this.fieldMappingQueryDef[err['field']]} is invalid`;
        }).join('; ') + '.';
      }

      resetErrors() {
        var key, ref, results, val;
        this.$scope.errorMessage = '';
        this.required_fields = {};
        ref = this.configForm;
        results = [];
        for (key in ref) {
          val = ref[key];
          if (typeof val === 'object' && val.hasOwnProperty('$modelValue')) {
            results.push(val.$setValidity('valid', true));
          } else {
            results.push(void 0);
          }
        }
        return results;
      }

      resetError(field) {
        return this.configForm[field].$setValidity('valid', true);
      }

      setErrors(errors, description) {
        if (errors) {
          this.$scope.errorMessage = this.getUserFriendlyErrorMessages(errors);
          this.setInvalidFields(errors);
          return this.setRequiredFields(errors);
        } else {
          return this.$scope.errorMessage = description;
        }
      }

      getAvaliableHttpMethods(http_methods) {
        var available_methods, b, m;
        available_methods = {};
        for (m in http_methods) {
          if (!hasProp.call(http_methods, m)) continue;
          b = http_methods[m];
          if (b) {
            available_methods[m] = true;
          }
        }
        return available_methods;
      }

      getPayload(raw_data) {
        var payload;
        payload = Object.assign({}, raw_data);
        delete payload.editable;
        delete payload.executable;
        delete payload.viewable;
        delete payload.url_template;
        delete payload.create;
        return payload;
      }

      save(execute = false) {
        var dirtyUrl, payload, promise;
        this.savingInProgress = true;
        if (!this.configData.read) {
          this.configData.read = [];
        }
        if (!this.configData.write) {
          this.configData.write = [];
        }
        if (!this.configData.execute) {
          this.configData.execute = [];
        }
        this.resetErrors();
        payload = this.getPayload(this.configData);
        if (this.$scope.newConfig) {
          promise = this.slashdb.createQueryDef(payload);
          promise.then((response) => {
            this.saveExecuteForm(execute);
            this.goToNewEndpoint();
            this.setRequiredFields([]);
          }).catch((response) => {
            this.setErrors(response.data.errors, response.data.description);
          });
        } else {
          dirtyUrl = this.configForm.query_id.$dirty;
          promise = this.slashdb.updateQueryDef(this.$scope.configId, payload);
          promise.then((response) => {
            this.blinkOK();
            this.$scope.configId = this.configData.query_id;
            this.updateConfigForm();
            if (dirtyUrl) {
              this.saveExecuteForm(execute);
              this.goToNewEndpoint();
            }
            this.setRequiredFields([]);
          }).catch((response) => {
            this.setErrors(response.data.errors, response.data.description);
          });
        }
        promise.finally(() => {
          this.savingInProgress = false;
        }).finally(() => {
          this.sessionControlService.onSessionLost(() => {
            return this.setErrors(null, this.sessionControlService.expiredMsg);
          });
          return this.sessionControlService.onSessionRestore(() => {
            // update user
            this.slashdb.setupSettings();
            return this.resetErrors();
          });
        });
        return promise;
      }

      goToNewEndpoint() {
        return this.$window.location.assign(`/querystudio/${this.configData.query_id}`);
      }

      saveAndExecute() {
        var execute;
        return this.save(execute = true).then((sdbResponse) => {
          if (!this.$scope.newConfig && sdbResponse.status >= 200 && sdbResponse.status < 400) {
            return this.updateConfigForm().then(() => {
              var res;
              res = this.executeQuery();
              if (!res) {
                return this.switchTab('bottom', 'execute');
              }
            });
          }
        });
      }

      isPrevPageEnabled() {
        return !(this.resultsResponseCode >= 400 || this.offset <= 0 || this.executionInProgress || this.savingInProgress);
      }

      previousResultPage() {
        this.offset -= this.limit;
        if (this.offset < 0) {
          this.offset = 0;
        }
        this.executeQuery();
      }

      isNextPageEnabled() {
        return !(this.resultsResponseCode >= 400 || !this.queryOutput || this.getQueryOutputSize() < this.limit || this.executionInProgress || this.savingInProgress);
      }

      nextResultPage() {
        this.offset += this.limit;
        this.executeQuery();
      }

      updateExecUrl() {
        var key, url, urlPath;
        if (this.configData) {
          this.checkParamNullState();
          this.validate();
          if (this.ready) {
            for (key in this.execUrls) {
              urlPath = this.QES.getExecPath(this.configData.query_id, this.queryParameters, this.limit, this.offset, this.count, key, this.distinct, this.transpose, this.stream, this.nullStr, this.mapToSortArray(this.sortMap));
              this.execUrls[key] = this.QES.queryBasePath + urlPath;
            }
          } else {
            for (key in this.execUrls) {
              this.execUrls[key] = '';
            }
          }
          url = this.QES.getExecPath(this.configData.query_id, this.queryParameters, this.limit, this.offset, this.count, this.format, this.distinct, this.transpose, this.stream, this.nullStr, this.mapToSortArray(this.sortMap));
          this.execUrl = this.QES.queryBasePath + url;
          this.execPath = url;
          return this.execUrl;
        }
      }

      setParamNull(paramKey) {
        if (this.parameterNullState[paramKey] === true) {
          this.parameterNullState[paramKey] = false;
          return this.queryParameters[paramKey] = '';
        } else {
          this.parameterNullState[paramKey] = true;
          return this.queryParameters[paramKey] = this.slashdb.settings['sqlpassthru.null_str'];
        }
      }

      checkParamNullState() {
        var key, results, results1;
// check if any of the parameters are currently set to the null placeholder typed in as a string
// if so, use nullStr query parameter string to define a custom placeholder
        for (key in this.queryParameters) {
          if (this.queryParameters[key] === this.slashdb.settings['sqlpassthru.null_str'] && this.parameterNullState[key] !== true) {
            this.nullStr = true;
            break;
          }
          this.nullStr = false;
        }
        // when null placeholder value is entered as a string:
        // pass over existing nulls and set them to the custom placeholder
        if (this.nullStr === true) {
          results = [];
          for (key in this.queryParameters) {
            if (this.parameterNullState[key] === true) {
              results.push(this.queryParameters[key] = this.nullPlaceHolder);
            } else {
              results.push(void 0);
            }
          }
          return results;
        // if an existing string value containing the null placeholder is changed to something else:
        // pass over existing nulls and set them to default placeholder
        } else if (this.nullStr === false) {
          results1 = [];
          for (key in this.queryParameters) {
            if (this.parameterNullState[key] === true) {
              results1.push(this.queryParameters[key] = this.slashdb.settings['sqlpassthru.null_str']);
            } else {
              results1.push(void 0);
            }
          }
          return results1;
        }
      }

      executeQuery() {
        var queryParams, queryStringParams, ref;
        this.validate();
        if (this.ready) {
          queryStringParams = {
            limit: this.limit,
            offset: this.offset,
            columns: true,
            numeric_columns: true
          };
          if (this.distinct) {
            queryStringParams.distinct = '';
          }
          if (this.transpose) {
            queryStringParams.transpose = '';
          }
          if (this.stream) {
            queryStringParams.stream = true;
          }
          if (this.sortMap.size) {
            queryStringParams.sort = this.mapToSortArray(this.sortMap);
          }
          if (this.count) {
            queryStringParams['count'] = '';
          }
          if (this.nullStr) {
            queryStringParams['nullStr'] = this.nullPlaceHolder;
          }
          if (Object.keys(this.queryParameters).length === 0) {
            queryParams = null;
          } else if ((ref = this.selectedHTTPMethod) === 'PUT' || ref === 'POST') {
            queryParams = [this.queryParameters];
          } else {
            queryParams = this.queryParameters;
          }
          this.executionInProgress = true;
          this.QES.executeQuery(this.configData.query_id, this.selectedHTTPMethod, queryParams, queryStringParams).finally(() => {
            this.sessionControlService.onSessionLost(() => {
              return this.resultsResponseWarning = this.sessionControlService.expiredMsg;
            });
            return this.sessionControlService.onSessionRestore(() => {
              this.queryOutput = null;
              this.resultsResponseCode = void 0;
              this.resultsResponseMessage = void 0;
              this.resultsResponseWarning = void 0;
              this.resultsCount = void 0;
              return this.resultsCountFailed = void 0;
            });
          });
        }
        return this.ready;
      }

      validate() {
        var formField, key, ref, ref1, ref2, valid, value;
        valid = true;
        valid = valid && this.limit > -1;
        valid = valid && (ref = this.format, indexOf.call(this.extensions, ref) >= 0);
        valid = valid && (ref1 = this.selectedHTTPMethod, indexOf.call(this.httpMethods, ref1) >= 0);
        if (this.executeForm) {
          for (key in this.queryParameters) {
            formField = this.executeForm[key];
            if (formField) {
              formField.$touched = true;
              valid = valid && formField.$valid;
            }
          }
        } else {
          ref2 = this.queryParameters;
          // page not loaded yet, just check that params are present
          for (key in ref2) {
            value = ref2[key];
            valid = valid && value;
          }
        }
        return this.ready = valid;
      }

      blinkOK() {
        this.$scope.okVisible = true;
        this.$timeout(() => {
          this.$scope.okVisible = false;
        }, 1000);
      }

      updateSuggestedMethods() {
        var i, searchTerm, sqlstr, suggestedMethods;
        suggestedMethods = {
          'GET': false,
          'POST': false,
          'PUT': false,
          'DELETE': false
        };
        if (this.configData === null) {
          return suggestedMethods;
        }
        sqlstr = this.configData.sqlstr;
        searchTerm = void 0;
        i = 0;
        while (i < methodsMappingKeysL) {
          searchTerm = methodsMapping[methodsMappingKeys[i]];
          suggestedMethods[searchTerm.method] = searchTerm.re.exec(sqlstr) !== null;
          i++;
        }
        return this.$scope.suggestedMethods = suggestedMethods;
      }

      getSqlstrParams(sqlstr) {
        var allParams, allParamsRegex, excludedParams, excludedParamsRegex, match, paramName;
        allParams = [];
        allParamsRegex = /:([\w\$]+)(?![:\w\$])/g;
        match = allParamsRegex.exec(sqlstr);
        while (match) {
          paramName = match[1];
          allParams.push(paramName);
          match = allParamsRegex.exec(sqlstr);
        }
        excludedParams = [];
        excludedParamsRegex = /[:\w\$\x5c]:([\w\$]+)(?![:\w\$])/g;
        match = excludedParamsRegex.exec(sqlstr);
        while (match) {
          paramName = match[1];
          excludedParams.push(paramName);
          match = excludedParamsRegex.exec(sqlstr);
        }
        return $(allParams).not(excludedParams).get();
      }

      updateExecuteForm(query) {
        var availableMethods, j, l, len, len1, len2, len3, m, n, o, oldParameters, paramName, paramValue, ref, ref1, ref2, ref3, ref4, ref5, v;
        oldParameters = angular.copy(this.queryParameters);
        availableMethods = (function() {
          var ref, results;
          ref = query.http_methods;
          results = [];
          for (m in ref) {
            v = ref[m];
            if (v === true) {
              results.push(m);
            }
          }
          return results;
        })();
        this.updateSuggestedMethods();
        this.queryParameters = {};
        // only when user have view access
        // params from editor
        if (query.sqlstr) {
          ref = this.getSqlstrParams(query.sqlstr);
          for (j = 0, len = ref.length; j < len; j++) {
            paramName = ref[j];
            if (oldParameters.hasOwnProperty(paramName)) {
              paramValue = oldParameters[paramName];
            } else {
              paramValue = '';
            }
            this.queryParameters[paramName] = paramValue;
          }
        }
        // when user do not have view access
        // params from API
        if (query.parameters) {
          ref1 = query.parameters;
          for (l = 0, len1 = ref1.length; l < len1; l++) {
            paramName = ref1[l];
            if (oldParameters.hasOwnProperty(paramName)) {
              paramValue = oldParameters[paramName];
            } else {
              paramValue = '';
            }
            this.queryParameters[paramName] = paramValue;
          }
        }
        // preselect some http method for the form. First select by suggested then by available.
        if (ref2 = this.selectedHTTPMethod, indexOf.call(availableMethods, ref2) < 0) {
          ref3 = this.httpMethods;
          for (n = 0, len2 = ref3.length; n < len2; n++) {
            m = ref3[n];
            if (indexOf.call(availableMethods, m) >= 0) {
              if (this.$scope.suggestedMethods[m]) {
                this.selectedHTTPMethod = m;
                break;
              }
            }
          }
        }
        if (ref4 = this.selectedHTTPMethod, indexOf.call(availableMethods, ref4) < 0) {
          ref5 = this.httpMethods;
          for (o = 0, len3 = ref5.length; o < len3; o++) {
            m = ref5[o];
            if (indexOf.call(availableMethods, m) >= 0) {
              this.selectedHTTPMethod = m;
              break;
            }
          }
        }
      }

      newQuery() {
        if (this.configForm.$dirty || this.sqlDirty) {
          return this.confirmNewQueryVisible = true;
        } else {
          return this.openNewQueryPage();
        }
      }

      openNewQueryPage() {
        return window.location.assign("/querystudio/");
      }

      sqlKeyDown(e) {
        if (this.configData.executable && (e.ctrlKey || e.metaKey) && (e.keyCode === 13 || e.keyCode === 10)) {
          return this.saveAndExecute();
        }
      }

      sqlChange(e) {
        if (this.configData.executable) {
          this.updateExecuteForm(this.configData);
          return this.sqlDirty = true;
        }
      }

      onHttpMethodChange() {
        if (this.configData.executable) {
          this.configData.http_methods = this.getAvaliableHttpMethods(this.configData.http_methods);
          return this.updateExecuteForm(this.configData);
        }
      }

      onSort(column) {
        var v;
        if (this.executionInProgress) {
          return;
        }
        if (column === null) {
          this.sortMap.clear();
        } else if (!this.sortMap.has(column)) {
          this.sortMap.set(column, true);
        } else {
          v = this.sortMap.get(column);
          if (v) {
            this.sortMap.set(column, false);
          } else {
            this.sortMap.set(column, true);
          }
        }
        return this.executeQuery();
      }

      mapToSortArray(sortMap) {
        return Array.from(sortMap, function([k, v]) {
          if (v) {
            return k;
          } else {
            return `-${k}`;
          }
        }).join(',');
      }

      getSortCssClass(column) {
        var v;
        if (this.sortMap.has(column)) {
          v = this.sortMap.get(column);
          if (v) {
            return '-up';
          } else {
            return '-down';
          }
        } else {
          return '';
        }
      }

      getQueryOutputSize() {
        var col, error;
        if (this.transpose) {
          col = this.queryOutput.columnNames[0];
          try {
            return this.queryOutput.data[col].length;
          } catch (error1) {
            error = error1;
            return 0;
          }
        } else {
          return this.queryOutput.data.length;
        }
      }

      getResultElementClass(value, col) {
        var classes;
        classes = "";
        if (this.queryOutput.numericColumns[col]) {
          classes += "numeric-val ";
        }
        if (value === null) {
          classes += "null-val ";
        }
        if (value === "") {
          classes += "empty-val ";
        }
        return classes.trim();
      }

      getResultValue(value) {
        var decimalDigits;
        decimalDigits = this.slashdb.settings['html_sdb.float_decimal_digits'];
        if (value === null) {
          return this.slashdb.settings['html_sdb.sqlpassthru.null_display_str'];
        }
        if (value === "") {
          return this.slashdb.settings['html_sdb.sqlpassthru.empty_display_str'];
        }
        // check if integer or float
        if (Number(decimalDigits) === decimalDigits && decimalDigits % 1 === 0 && Number(value) === value && value % 1 !== 0) {
          return Math.round(value * Math.pow(10, decimalDigits)) / Math.pow(10, decimalDigits);
        } else {
          return value;
        }
      }

    };

    QuerystudioController.$inject = ['$window', '$location', '$timeout', '$rootScope', '$scope', 'slashDB', 'QueryExecService', 'selectizeService', 'sessionControlService'];

    methodsMapping = {
      'select': {
        re: /\bselect\b/i,
        method: 'GET'
      },
      'insertOrExec': {
        re: /\binsert|exec\b/i,
        method: 'POST'
      },
      'update': {
        re: /\bupdate\b/i,
        method: 'PUT'
      },
      'delete': {
        re: /\bdelete\b/i,
        method: 'DELETE'
      }
    };

    methodsMappingKeys = Object.keys(methodsMapping);

    methodsMappingKeysL = methodsMappingKeys.length;

    return QuerystudioController;

  }).call(this);

  angular.module('sdbgui').component('querystudio', {
    templateUrl: '/static/components/sdbgui-querystudio/querystudio.html',
    controller: QuerystudioController,
    bindings: {
      queryId: '@'
    }
  });

}).call(this);
