angular.module('sdbgui')
    .component('sqlpassthru', {
        templateUrl: '/static/components/sdbgui-sqlpassthru/sdbgui-sqlpassthru.template.html',
        controller: ['$scope', '$location', 'sortingService', 'slashDB', function QueryController($scope, $location, sortingService, slashdb) {
            var ctrl = this;
            ctrl.sort = sortingService;

            ctrl.queryDict = {};
            ctrl.selectedQueryParams = {};
            ctrl.queryOutput = {
                columnNames: [],
                data: []
            };
            ctrl.executeBaseUrl = '';
            ctrl.loadMoreVisible = false;

            slashdb.getQueries(true)
                .then(function (response) {
                    var data = response.data[0],
                        queryList = [],
                        query;

                    Object.keys(data).forEach(function (queryId) {
                        query = data[queryId];
                        paramsDict = {};
                        query.paramsDict = paramsDict;
                        query.parameters.forEach(function (paramName) {
                            paramsDict[paramName] = '';
                        });
                        queryList.push(query);
                    });

                    ctrl.listAlert = '';
                    ctrl.queryList = queryList;
                })
                .catch(function (response) {
                    ctrl.listAlert = response.headers('Warning');
                    ctrl.queryList = [];
                });
        }]
    });

angular.module('sdbgui')
    .directive('executeQuery', ['$location', 'slashDB', function ($location, slashdb) {
        return {
            restrict: 'E',
            templateUrl: '/static/components/sdbgui-sqlpassthru/sdbgui-sqlpassthru-execute.template.html',
            scope: {
                query: '='
            },
            link: function (scope, element, attrs) {
                var executeOffset = 0;
                var extensions = ['html', 'json', 'csv', 'xml', 'xsd'];
                var queryBasePath = $location.protocol() + '://' + $location.host();
                var defaultLimit = parseInt(slashdb.settings.html_limit) || 29;
                if ($location.port && $location.port != 80) {
                    queryBasePath = queryBasePath + ':' + $location.port();
                }
                scope.queryBasePath = queryBasePath + '/query';
                scope.queryOutput = {
                    columnNames: [],
                    data: []
                };
                scope.queryExecuted = false;

                scope.httpMethods = ['GET', 'POST', 'PUT', 'DELETE'];
                scope.selectedHTTPMethod = 'GET';

                scope.$watch('query', function () {
                    if (scope.query != null) {
                        scope.updateQueryUrls();
                        clearOutput();

                        // init selectedHTTPMethod with the first active method
                        scope.selectedHTTPMethod = function () {
                            if (scope.query != null) {
                                var httpMethods = scope.httpMethods,
                                    methodName;
                                for (var i = 0, len_ = httpMethods.length; i < len_; i++) {
                                    methodName = httpMethods[i];
                                    if (scope.query.http_methods[methodName]) {
                                        return methodName;
                                    }
                                }
                            }
                            return 'GET';
                        }();
                    }
                });

                scope.updateQueryUrls = function (event) {
                    // don't update od enter,
                    // a submit will trigger scope.executeQuery
                    // and an update will brake thing (i.e. no data msg)
                    if (event != null && event.key == 'Enter') {
                        return;
                    }

                    scope.queryExecuted = false;
                    // url escaping information
                    var query = scope.query,
                        substituteInfo = [],
                        urls = [],
                        showEscapeInfo = false,
                        executeBaseUrl = '/' + query.query_id,
                        keys = Object.keys(query.paramsDict),
                        kl = keys.length,
                        showUrl = kl === 0,
                        escapedValue, key, value, exl, idx, url;

                    for (idx = 0; idx < kl; idx++) {
                        key = keys[idx];
                        value = query.paramsDict[key];
                        if (value) {
                            escapedValue = slashdb.escapeValue(value);
                            showEscapeInfo = showEscapeInfo || (escapedValue != value);
                            executeBaseUrl = executeBaseUrl + '/' + key + '/' + escapedValue;
                            if (escapedValue || showUrl) {
                                showUrl = true;
                            }
                        }
                    }

                    if (showUrl) {
                        query.executeUrl = executeBaseUrl;
                        for (idx = 0, exl = extensions.length; idx < exl; idx++) {
                            url = executeBaseUrl + '.' + extensions[idx];
                            if (extensions[idx] != 'xsd') {
                                url = url + '?limit=' + defaultLimit;
                            }
                            urls.push(url);
                        }
                    }
                    for (key in slashdb.settings.reversed_url_substitution) {
                        substituteInfo.push(key + ' is replaced with ' + slashdb.settings.reversed_url_substitution[key]);
                    }

                    scope.showEscapeInfo = showEscapeInfo;
                    scope.substituteInfo = substituteInfo.join(', ');
                    query.urls = urls;
                };

                scope.close = function () {
                    scope.query = null;
                };

                scope.executeQuery = function () {
                    scope.queryExecuted = false;
                    clearOutput();
                    var valid = true,
                        query = scope.query;

                    for (var key in query.paramsDict) {
                        scope.queryParamsFormCtrl[key].$setTouched();
                        valid = valid && scope.queryParamsFormCtrl[key].$valid;
                    }

                    executeOffset = 0;
                    if (valid) {
                        scope.executeAlert = '';
                        var promise;
                        var executeUrl = query.executeUrl + '.json';
                        switch (scope.selectedHTTPMethod) {
                            case 'POST':
                                promise = slashdb.executeQuery(
                                    executeUrl, scope.selectedHTTPMethod, query.paramsDict
                                );
                                break;
                            case 'PUT':
                                promise = slashdb.executeQuery(
                                    executeUrl, scope.selectedHTTPMethod, query.paramsDict
                                );
                                break;
                            case 'DELETE':
                                promise = slashdb.executeQuery(executeUrl, 'DELETE');
                                break;
                            default:
                                promise = slashdb.executeQuery(executeUrl, 'GET', null, {
                                    params: {
                                        limit: defaultLimit
                                    }
                                });
                        }
                        promise
                            .then(function (response) {
                                scope.queryExecuted = true;
                                if (response.data.length > 0) {
                                    scope.queryOutput = {
                                        columnNames: Object.keys(response.data[0]),
                                        data: response.data
                                    };
                                    scope.loadMoreVisible = true;
                                }
                            })
                            .catch(function (response) {
                                clearOutput();
                                scope.executeAlert = response.headers('Warning');
                            });
                    }
                };

                scope.loadMore = function () {
                    executeOffset += defaultLimit;
                    scope.executeAlert = '';
                    slashdb.executeQuery(scope.query.executeUrl, scope.selectedHTTPMethod, scope.query.paramsDict, {
                            params: {
                                limit: defaultLimit,
                                offset: executeOffset
                            }
                        })
                        .then(function (response) {
                            if (response.data.length > 0) {
                                response.data.forEach(function (v) {
                                    scope.queryOutput.data.push(v);
                                });
                            } else {
                                scope.loadMoreVisible = false;
                                scope.executeAlert = 'No more data.';
                            }
                        })
                        .catch(function (response) {
                            if (response.status == 404) {
                                scope.loadMoreVisible = false;
                                scope.executeAlert = 'No more data.';
                            } else {
                                scope.executeAlert = response.headers('Warning');
                            }
                        });
                };

                function clearOutput() {
                    scope.queryOutput = {
                        columnNames: [],
                        data: []
                    };
                }
            }
        };
    }]);