'use strict';

angular.module('sdbgui').component('userconfigs', {
    templateUrl: '/static/components/sdbgui-userconfigs/sdbgui-userconfigs.template.html',
    controller: ['$scope', '$uibModal', 'sortingService', 'slashDB', 'sessionControlService',
        function DbConfigController($scope, $uibModal, sortingService, slashdb, sessionControlService) {
            var ctrl = this;
            ctrl.sort = sortingService;

            ctrl.$onInit = function () {
                $scope.searchable_fields = ['user_id', 'name'];
                ctrl.addConfigVisible = false;
                ctrl.updateConfigList();
            };

            ctrl.updateConfigList = function () {
                slashdb.get('/userdef.json', {
                        params: {
                            guidata: ''
                        }
                    })
                    .then(function (response) {
                        var data = response.data[0],
                            configs = [];
                        angular.forEach(data.userdefs, function (userConf) {
                            if (userConf.viewable || userConf.editable){
                                configs.push(userConf);
                            }
                        });
                        ctrl.listAlert = '';
                        ctrl.configs = configs;
                        ctrl.addConfigVisible = data.create_user;
                    }) // can't the previous request (with guidata) raise this error?
                    .then(function (response) {
                        return slashdb.get('/userdef.json')
                    })
                    .catch(function (response) {
                        ctrl.listAlert = response.headers('Warning');
                        ctrl.configs = [];
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            ctrl.listAlert = sessionControlService.expiredMsg
                        )

                        sessionControlService.onSessionRestore(() => {
                            ctrl.updateConfigList();
                        })
                    })

            };

            ctrl.openModal = function (userId, editable) {
                var modalInstance = $uibModal.open({
                    templateUrl: '/static/components/sdbgui-userconfigs/sdbgui-userconfigs-modal.template.html',
                    size: 'lg',
                    backdrop: 'static',
                    controller: 'UserConfigModalCtrl',
                    resolve: {
                        configId: function () {
                            return userId;
                        },
                        editable: function () {
                            return editable;
                        },
                        addNewBtnMsg: function () {
                            return '';
                        }
                    }
                });
                modalInstance.result
                    // refresh config list after closing modal
                    .catch(function () {
                        ctrl.updateConfigList();
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            ctrl.listAlert = sessionControlService.expiredMsg
                        )

                        sessionControlService.onSessionRestore(() => {
                            ctrl.updateConfigList()
                        })
                    });
            };
        }]
    })
    .controller('UserConfigModalCtrl',
    ['$timeout', '$scope', '$q', '$uibModalInstance', '$http', 'slashDB', 'configId',
        'editable', 'addNewBtnMsg', 'selectizeService', 'looseAccess', 'sessionControlService',
        function ($timeout, $scope, $q, $uibModalInstance, $http, slashdb, configId,
                  editable, addNewBtnMsg, selectizeService, looseAccess, sessionControlService) {
            /**
             * @typedef {Object} TestConnection
             * @property {boolean} connected
             * @property {boolean} loading
             * @property {string} error
             * @property {string} cssClass
             */

            /**
             * @typedef {Object} DBMapping
             * @property {string} dbId
             * @property {string} dbuser
             * @property {string} dbpass
             * @property {TestConnection} connectionStatus
             */

            /** @type {string[]} */
            var _allDatabases = [];
            let ctrl = this

            ctrl.$onInit = function () {
                $scope.slashdbSettings = slashdb.settings;
                $scope.selectizeService = selectizeService;
                selectizeService.updateUserOptions(null);
                $scope.addNewBtnMsg = '';
                $scope.newConfig = !Boolean(configId);
                $scope.configId = configId;
                // stores database definition model, some initial values
                $scope.configData = {
                    view: [],
                    edit:[],
                };
                $scope.extraConfigData = {
                    /** @type {DBMapping[]} */
                    dbMappings: [],
                }; // stores values only for local use, not to be sent to slashdb
                $scope.okVisible = false;
                $scope.showPermissionSection = configId != 'admin';
                $scope.modalAlert = '';
                $scope.warningModal = '';
                $scope.editable = editable || $scope.newConfig;
                $scope.selfEdit = slashdb.settings.user === $scope.configId;
                $scope.deleteVisible = Boolean(configId) && $scope.editable &&
                    !$scope.selfEdit;
                $scope.loadConfig().then(function() {
                    _allDatabases = slashdb.settings.all_databases;
                });
                $scope.addNewBtnMsg = addNewBtnMsg;
                $scope.required_fields = {}
                $scope.savingInProgress = false
            };

            /**
             * @property {string} dbId
             * @return {string[]}
             */
            $scope.allDatabases = function (currentDBId) {
                /** @type {string[]} */
                var dbMappingIds = ($scope.extraConfigData.dbMappings || []).map(function (item) {
                    return item.dbId;
                });
                return _allDatabases.filter(function (dbId) {
                    return dbId == '' || dbId == currentDBId || dbMappingIds.indexOf(dbId) < 0;
                });
            };

            $scope.loadConfig = function () {
                $scope.newConfig = !$scope.configId;
                if ($scope.newConfig) {
                    return $q(function (resolve, reject) {
                        resolve();
                    });
                }

                return slashdb.get('/userdef/' + $scope.configId + '.json', {
                        params: {
                            guidata: ''
                        }
                    })
                    .then(function (response) {
                        var configData = response.data[0],
                            allowedToEdit = ['admin', configData.creator].concat(configData.edit);

                        $scope.limitedEdit =
                            slashdb.settings.user !== 'admin' &&
                            slashdb.settings.user === $scope.configId &&
                            allowedToEdit.indexOf(slashdb.settings.user) < 0;

                        if (!$scope.configId) {
                            $scope.deleteVisible = false;
                        } else if (slashdb.settings.user === 'admin') {
                            $scope.deleteVisible = !$scope.selfEdit;
                        } else {
                            // self delete forbidden
                            $scope.deleteVisible = $scope.editable &&
                                !$scope.limitedEdit && !$scope.selfEdit
                        }

                        /** @type {DBMapping[]} */
                        var dbMappings = [];
                        /** @type {DBMapping} */
                        var dbMapping;
                        var dbs = response.data[0].databases,
                            dbId,
                            dbIds = Object.keys(dbs),
                            extraConfigData = {
                                disabledId: true,
                                viewDbList: configData.dbdef.indexOf('view') >= 0,
                                createDb: configData.dbdef.indexOf('create') >= 0,
                                viewUserList: configData.userdef.indexOf('view') >= 0,
                                createUser: configData.userdef.indexOf('create') >= 0,
                                viewQueryList: configData.querydef.indexOf('view') >= 0,
                                createQuery: configData.querydef.indexOf('create') >= 0,
                                dbMappings: dbMappings
                            };

                        for (var idx = 0, dbIdsL = dbIds.length; idx < dbIdsL; idx++) {
                            dbId = dbIds[idx];
                            dbMapping = dbs[dbId];
                            dbMappings.push({
                                dbId: dbId,
                                dbuser: dbMapping.dbuser,
                                dbpass: dbMapping.dbpass
                            });
                        }

                        selectizeService.updateUserOptions(configData);
                        $scope.configData = configData;
                        $scope.extraConfigData = angular.merge({}, $scope.extraConfigData, extraConfigData);

                        // refresh slashDB.settings
                        return slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            $scope.modalAlert = sessionControlService.expiredMsg
                        )

                        sessionControlService.onSessionRestore(() => {
                            $scope.loadConfig();
                            $scope.modalAlert = '';
                            $scope.warningModal = '';
                        })
                    });
            };

            $scope.initNewDBMapping = function () {
                $scope.addNewBtnMsg = '';
                $scope.extraConfigData.dbMappings.push({
                    dbId: '',
                    dbuser: '',
                    dbpass: '',
                });
            };

            $scope.delete = function () {
                slashdb.deleteUserDef($scope.configId)
                    .then(function (response) {
                        $uibModalInstance.dismiss();

                        // refresh slashDB.settings
                        slashdb.setupSettings();
                    })
                    .catch(function (response) {
                        $scope.modalAlert = response.headers('Warning');
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() => {
                            $scope.modalAlert = sessionControlService.expiredMsg;
                        })

                        sessionControlService.onSessionRestore(() => {
                            // refresh current user
                            slashdb.setupSettings()
                            $scope.modalAlert = '';
                            $scope.warningModal = '';
                        })
                    });
            };

            var fieldMappingUserDef = {
                'user_id': 'User ID',
                'password': 'Password',
                'api_key': 'API Key',
                'name': 'Full Name',
                'email': 'E-mail',
                'view': 'View',
                'edit': 'Edit',
                'databases': 'Database Mappings',
                'dbdef': 'Databases',
                'userdef': 'Users',
                'querydef': 'Sql Pass-Thru Queries',
            };

            var setInvalidFields = function (errors) {
                errors.forEach(function (err, _, __) {
                    if (err['type'] === 'INVALID') {
                        if (err['field'] === 'user_id' &&
                        $scope.configData.user_id === '') {
                            return
                        }
                        $scope.configForm[err['field']].$setValidity('valid', false)
                    }
                });
            };

            var setRequiredFields = function(errors) {
                var required = {};
                errors.forEach(function (err, _, __) {
                    if (err['type'] === 'MISSING') {
                        required[err['field']] = true
                    }
                });

                ['user_id'].forEach(function (value) {
                    required[value] = true
                });
                $scope.required_fields = required
            };

            var getUserFriendlyErrorMessages = function (errors) {
                let msg = 'Errors found in incoming user definition: ' +
                    errors.map(
                        function (err, _, __) {
                            switch (err['type']) {
                                case 'MISSING':
                                    return fieldMappingUserDef[err['field']] + ' is obligatory'
                                case 'EXTRA':
                                    return err['description'] + " - try to reload page"
                                case 'INVALID':
                                default:
                                    switch(err['field']) {
                                        case 'user_id':
                                            if (err['description'].includes('already exists')) {
                                                return 'User already exists'
                                            }
                                            if ($scope.configData.user_id === '') {
                                                return fieldMappingUserDef[err['field']] + ' is obligatory'
                                            }
                                        case 'password':
                                        case 'old_password':
                                        case 'api_key':
                                            return err['description']
                                        case 'view':
                                            return '"View": ' + err['description']
                                        case 'Edit':
                                            return '"Edit" ' + err['description']
                                        default:
                                            if (err['description'].includes('is too long')) {
                                                return fieldMappingUserDef[err['field']] + ' is too long'
                                            }
                                            return fieldMappingUserDef[err['field']] + ' is invalid'
                                    }
                            }
                        }
                    ).join(', ')

                if (!msg.endsWith('.')) {
                    msg = msg + '.'
                }
                return msg
            };

            var resetErrors = function() {
                $scope.modalAlert = '';
                $scope.warningModal = '';
                for (var key in $scope.configForm) {
                    var value = $scope.configForm[key];
                    if (typeof value === 'object'
                        && value.hasOwnProperty('$modelValue')) {
                        value.$setValidity('valid', true)
                        value.$iswarning = false
                    }
                }
                $scope.required_fields = {}
            };

            $scope.resetError = function (field) {
                  $scope.configForm[field].$setValidity('valid', true)
            };

            var setErrors = function(errors, description) {
                if (errors !== undefined && errors.length > 0){
                    $scope.modalAlert = getUserFriendlyErrorMessages(errors);
                    setRequiredFields(errors);
                    setInvalidFields(errors);
                } else {
                    $scope.modalAlert = description;
                }
            };

            var setWarnings = function(response) {
                var msg = ""
                if (typeof response.data.warnings !== 'undefined') {
                    if (response.data.warnings.length > 0) {
                        msg = response.data.warnings[0].description
                        $scope.configForm["email"].$iswarning = true
                    }
                }
                else if (response.headers("Warning")){
                    msg = response.headers("Warning")
                    $scope.configForm["email"].$iswarning = true
                }
                $scope.warningModal = msg
            };

            $scope.save = function () {
                $scope.savingInProgress = true;
                var ecd = $scope.extraConfigData;
                var uploadData = $scope.configData;
                var dbdef = [];
                var userdef = [];
                var querydef = [];

                if (ecd.viewDbList) dbdef.push('view');
                if (ecd.createDb) dbdef.push('create');
                if (ecd.viewUserList) userdef.push('view');
                if (ecd.createUser) userdef.push('create');
                if (ecd.viewQueryList) querydef.push('view');
                if (ecd.createQuery) querydef.push('create');

                uploadData.dbdef = dbdef;
                uploadData.userdef = userdef;
                uploadData.querydef = querydef;

                uploadData.databases = {};
                for (var i = 0; i < ecd.dbMappings.length; i++) {
                    var dbMapping = ecd.dbMappings[i];

                    if (dbMapping.dbId !== ''){
                        uploadData.databases[dbMapping.dbId] = {
                            dbuser: dbMapping.dbuser,
                            dbpass: dbMapping.dbpass
                        };
                    }

                }

                // when limitedEdit then remove database mappings to avoid an attempt to update
                if ($scope.limitedEdit){
                    delete uploadData.databases;
                }

                resetErrors();

                let promise;
                if ($scope.newConfig) {
                    promise = slashdb.createUserDef(uploadData)
                        .then(function (response) {
                            $scope.blinkOK();
                            $scope.configId = uploadData.user_id;
                            $scope.loadConfig();
                            $scope.configForm.$setPristine();
                            setWarnings(response)
                        })
                } else {
                    promise = slashdb.updateUserDef($scope.configId, uploadData)
                        .then(function (response) {
                            $scope.blinkOK();
                            $scope.loadConfig();
                            $scope.configForm.$setPristine();
                            setWarnings(response)
                        })
                }
                promise.catch(function (response) {
                        setErrors(response.data.errors, response.data.description)
                        setWarnings(response)
                    })
                    .finally(() => {
                        $scope.savingInProgress = false;

                        sessionControlService.onSessionLost(() =>
                            $scope.modalAlert = sessionControlService.expiredMsg
                        )

                        sessionControlService.onSessionRestore(() => {
                            // refresh current user
                            slashdb.setupSettings()
                            $scope.modalAlert = '';
                            $scope.warningModal = '';
                        })
                    });
            };

            $scope.close = function () {
                $uibModalInstance.dismiss();
            };

            $scope.checkOther = function (eventField, relatedFieldName) {
                if (eventField) $scope.extraConfigData[relatedFieldName] = true;
            };

            $scope.uncheckOther = function (eventField, relatedFieldName) {
                if (!eventField) $scope.extraConfigData[relatedFieldName] = false;
            };

            $scope.hasAccess = looseAccess.hasAccess;

            $scope.blinkOK = function () {
                $scope.okVisible = true;
                $timeout(function () {
                    $scope.okVisible = false;
                }, 1000);
            };

            let generateRandomKey = (n) => {
                const STEPLEN = n * 2;
                const MINLEN = n;

                function genString() {
                    let array = new Uint8Array(STEPLEN);
                    window.crypto.getRandomValues(array);
                     /* turn into non-typed array */
                    array = Array.apply([], array);
                    array = array.filter(function(x) {
                        /* strip non-printables: if we transform into desirable
                        range we have a propability bias, so I suppose we better
                        skip this character */
                        // 0-9 || a-z
                        return x >= 48 && x <= 57 || x >= 97 && x <= 122;
                    });
                    return String.fromCharCode.apply(String, array);
                }
                let key = genString();
                while (key.length < MINLEN) {
                    key += genString();
                }
                return key.slice(0, n);
            };

            $scope.setRandomApiKey = function () {
                if ($scope.configData != null) {
                    var randomKey = generateRandomKey(32),
                        apiKey = $scope.configData.api_key || '',
                        semicolonIdx = apiKey.indexOf(':');

                    semicolonIdx = semicolonIdx > -1 ? semicolonIdx + 1 : 0;
                    $scope.configData.api_key = apiKey.slice(0, semicolonIdx) + randomKey;
                }
            };

            $scope.checkDbMappingConnection = function (dbMappingIndex) {
                if (!$scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus) {
                    $scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus = {}
                }
                $scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus['loading'] = true;
                return $http.post(
                    '/settings/check-database-mapping-connection.json',
                    $scope.extraConfigData.dbMappings[dbMappingIndex],
                )
                    .then(function successCallback(response){
                        $scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus = {
                            connected: true,
                            loading: false,
                            error: '',
                            cssClass: 'connection-success',
                        }
                    })
                    .catch(function errorCallback(response){
                        $scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus = {
                            connected: false,
                            loading: false,
                            error: response.data.description ? response.data.description : 'Unknown error',
                            cssClass: 'connection-error',
                        };
                        return $q.reject();
                    })
                    .finally(function(){
                        $scope.modalAlert = $scope.extraConfigData.dbMappings[dbMappingIndex].connectionStatus.error;
                    });
            }
        }
    ]);
