// @ts-check

/**
 * @param {string} str
 * @param {string} [suffix=".html"]
 * @param {string[]} [ignoreList=["#"]]
 * @returns {string}
 */
var rmSuffix = function (str, suffix, ignoreList) {
    suffix = suffix || '.html';
    ignoreList = ignoreList || ['#']

    // filter-out chars that are in ignoreList
    str = str.split('').filter(function (char) {
        return ignoreList.indexOf(char) < 0;
    }).join('');

    var lenDiff = str.length - suffix.length;
    if (str.indexOf(suffix, lenDiff) !== -1) {
        return str.slice(0, lenDiff);
    }
    return str;
};

// @ts-ignore
angular.module('sdbgui')
    .directive('passwordMatch', function () {
        return {
            require: 'ngModel',
            scope: {
                otherModelValue: '=passwordMatch'
            },
            link: function ($scope, $elem, $attrs, $ctrl) {
                $ctrl.$validators.passwordMatch = function (modelValue) {
                    return modelValue == '' || modelValue == $scope.otherModelValue;
                };

                $scope.$watch("otherModelValue", function () {
                    $ctrl.$validate();
                });
            }
        };
    })

    .directive('fileModel', ['$parse', function ($parse) {
        return {
            restrict: 'A',
            require: 'ngModel',
            link: function (scope, element, attrs, ngModelCtrl) {
                element.bind('click', function () {
                    element[0].value = null;
                });
                element.bind('change', function () {
                    scope.$apply(function () {
                        ngModelCtrl.licError = null;
                        ngModelCtrl.$setViewValue(element[0].files[0]);
                    });
                });
            }
        };
    }])

    .directive('fileValidation', function () {
        return {
            require: 'ngModel',
            link: function (scope, el, attrs, ctrl) {
                ctrl.$setValidity('validFile', el.val() != '');
                //change event is fired when file is selected
                el.bind('change', function () {
                    ctrl.$setValidity('validFile', el.val() != '');
                    scope.$apply(function () {
                        ctrl.$setViewValue(el[0].files[0]);
                        ctrl.$render();
                    });
                });
            }
        };
    })

    .directive('licenseValidator', ['$q', 'slashDB', function ($q, slashdb) {
        return {
            require: 'ngModel',
            link: function (scope, element, attrs, ngModelCtrl) {
                ngModelCtrl.$asyncValidators.license = function (modelValue, viewValue) {
                    var def = $q.defer();
                    if (!modelValue) {
                        return $q.when();
                    }
                    slashdb.uploadLicense(modelValue, true)
                        .then(function (response) {
                            def.resolve();
                        })
                        .catch(function (response) {
                            ngModelCtrl.licError = response.headers('Warning');
                            def.reject();
                        });

                    return def.promise;
                };
            }
        };
    }])

    .directive('validInstanceId', ['$http', 'slashDB', function($http, slashdb){
        return {
            require: 'ngModel',
            link: function (scope, element, attrs, ngModelCtrl) {
                ngModelCtrl.$asyncValidators.validInstanceId = function(modelValue, viewValue){
                    if (slashdb.settings.ami_id){
                        return $http.post('/validate-instance-id', viewValue);
                    } else {
                        return true;
                    }
                }
            }
        }
    }])

    .directive('validConfig', [function(){
        return {
            require: ['ngModel', '^form'],
            scope: {
                validConfig: '='
            },
            link: function(scope, element, attrs, controllers){
                var ngModelCtrl = controllers[0],
                    formCtrl = controllers[1],
                    name = attrs.name;

                ngModelCtrl.$validators.config = function (modelValue) {
                    var configErrors = scope.validConfig;
                    for (var i = 0; i < configErrors.length; i++) {
                        var error = configErrors[i];

                        if (name == error.field && !error.hide){
                            if (error.type === 'INVALID') {
                                error.hide = true;
                                return false;
                            } else if (error.type === 'MISSING') {
                                return Boolean(modelValue);
                            } else {
                                return true;
                            }
                        }
                    }
                    return true;
                };

                scope.$watch('validConfig', function(){
                    formCtrl[name].$validate()
                })
            }
        }
    }])

    .directive('onlyAdmin', ['slashDB', function(slashdb){
        return {
            require: 'ngModel',
            link: function(scope, element, attrs, ngModelCtrl) {
                ngModelCtrl.$validators.onlyAdmin = function(modelValue, viewValue) {
                    var valid,
                        value = Boolean(viewValue),
                        isAdmin = slashdb.settings.user == 'admin';

                    valid = !(isAdmin && !value);

                    return valid;
                };

            }
        }
    }])

    .directive('withAccess', ['slashDB', 'looseAccess', function(slashdb, looseAccess){
        /*
        Field is valid if user is admin or it's a new query or current user is listed in the validated field.
        To prevent interfering with selectize directive (multiple scopes) queryId is acquired from attribute list.
         */
        return {
            require: 'ngModel',
            link: function(scope, element, attrs, ngModelCtrl) {
                scope.id = attrs.withAccess;
                ngModelCtrl.$validators.noAccess = function(modelValue, viewValue) {
                    return looseAccess.hasAccess(scope.id, viewValue, scope.newConfig)
                }
            }
        }
    }])

    .directive('ngEnter', function () {
        return function (scope, element, attrs) {
            element.bind("keydown keypress", function (event) {
                if (event.which === 13) {
                    scope.$apply(function () {
                        scope.$eval(attrs.ngEnter);
                    });

                    event.preventDefault();
                }
            });
        };
    })

    .directive('tooltip', [function ($compile) {
        return {
            restrict: 'E',
            scope: {},
            template: '<span ng-show="tooltipVisible"><span ng-transclude></span><span>',
            transclude: true,
            link: function (scope, element, attrs) {
                element.bind('click', function () {
                    scope.$apply(function () {
                        scope.tooltipVisible = !scope.tooltipVisible;
                    });
                });
            }
        };
    }])

    .directive('executeButton', ['$window', 'sessionControlService',
        function($window, sessionControlService) {
        return {
            restrict: 'E',
            scope: false,
            templateUrl: '/static/components/sdbgui-utils/executeButton.template.html',
            link: function ($scope, $element, $attrs){
                var queryStudioCtrl = $scope.$ctrl;

                $scope.formats = ['html', 'json', 'csv', 'xml', 'xsd']
                $scope.goTo = function(format) {
                    var url = queryStudioCtrl.execUrls[format];
                    if (url != '') {
                        if (queryStudioCtrl.executeOnly || !queryStudioCtrl.configData.editable){
                            queryStudioCtrl.validate();
                            if (queryStudioCtrl.ready) {
                                var resultWindow = $window.open('')
                                resultWindow.location.assign(url);
                                sessionControlService.onSessionRestore(() => {
                                    resultWindow.location.assign(url);
                                })
                            }
                        } else {
                            // todo save when only something changed
                            queryStudioCtrl.save()
                            .then(function(sdbResponse){
                                queryStudioCtrl.validate();
                                if (queryStudioCtrl.ready && sdbResponse.status >= 200 && sdbResponse.status < 400) {
                                    var resultWindow = $window.open('')
                                    resultWindow.location.assign(url);
                                    sessionControlService.onSessionRestore(() => {
                                        resultWindow.location.assign(url);
                                    })
                                }
                            });
                        }
                    }
                };

                $scope.isDisabled = function () {
                    let methods = $scope.$ctrl.configData.http_methods
                    return !(methods && 'GET' in methods && methods['GET'])
                }

                $scope.tooltipText = function () {
                    if ($scope.isDisabled()) {
                        return 'Available when GET method is selected.'
                    }
                    return ''
                }

                $scope.play = function(){
                    if (queryStudioCtrl.executeOnly || !queryStudioCtrl.configData.editable){
                        queryStudioCtrl.executeQuery()
                    } else {
                        queryStudioCtrl.saveAndExecute()
                    }
                };
                $scope.playButtonDisabled = () => {
                    return queryStudioCtrl.executionInProgress || queryStudioCtrl.savingInProgress;
                }
            }

        }

    }])

    .directive('dbSchema', ['$q', '$timeout', '$interval', 'slashDB', 'sessionControlService',
        function($q, $timeout, $interval, slashdb, sessionControlService){
        return {
            restrict: 'E',
            templateUrl: '/static/components/sdbgui-utils/dbschema.template.html',
            scope: {
                showSchema: '=',
                dbId: '='
            },
            link: function (scope, element, attrs) {
                scope.tables = [];

                scope.cleanSchema = function () {
                    return $q(function (resolve, reject) {
                        const sliceSize = 500;
                        scope.isLoading = true;
                        var interval = $interval(function () {
                            scope.tables.splice(0, sliceSize);

                            if (scope.tables.length == 0) {
                                $interval.cancel(interval);
                                scope.isLoading = false;
                                resolve();
                            }
                        }, 10);

                    })
                };
                scope.updateSchema = function (dbId) {
                    scope.schemaMessage = '';

                    // db id is not set
                    if (!dbId) {
                        scope.tables = [];
                        scope.schemaMessage = "Unable to display database schema because the database is not selected.";
                        return null;
                    }

                    var url = '/db/' + dbId + '.json?tables';
                    slashdb.get(url)
                        .then(function (response) {
                            const sliceSize = 500;
                            scope.isLoading = true;
                            scope.tables = response.data.slice(0, sliceSize);
                            var interval = $interval(function () {
                                scope.tables = response.data.slice(0, scope.tables.length + sliceSize);

                                if (scope.tables.length >= response.data.length) {
                                    $interval.cancel(interval);
                                    scope.isLoading = false;
                                }

                            }, 10);

                            return null;
                        })
                        .catch(function (response) {
                            scope.tables = [];
                            if (response.data.code === 503) {
                                scope.schemaMessage = "Unable to display database schema because the database is not connected.";
                            } else {
                                scope.schemaMessage = response.data.description;
                            }
                            return null
                        })
                        .finally(() => {
                                sessionControlService.onSessionLost(
                                    () => scope.schemaMessage = sessionControlService.expiredMsg
                                )

                                sessionControlService.onSessionRestore(
                                    () => scope.updateSchema(dbId)
                                )
                            }
                        );
                };

                scope.$watch('dbId', function (newValue, oldValue) {
                    if (newValue != oldValue || scope.tables.length == 0) {
                        scope.cleanSchema()
                            .then(function () {
                                scope.updateSchema(scope.dbId);
                            })
                    }
                });

                scope.$on('schema-control', function (event, allOpen) {
                    scope.isLoading = true;
                    var sliceStart = 0;
                    const sliceSize = 10;
                    var interval = $interval(function () {
                        for (var i=sliceStart; i < sliceStart + sliceSize; i++){
                            if (scope.tables[i] == undefined) {
                                $interval.cancel(interval);
                                scope.isLoading = false;
                                break;
                            } else {
                                scope.tables[i].open = allOpen
                            }
                        }

                        sliceStart += sliceSize;

                    }, 20);
                });
            }
        }
    }])

    .component('loading', {
        template: '{{ $ctrl.isLoading() ? $ctrl.whenLoading : $ctrl.whenFinished }}',
        controller: [function () {
            var ctrl = this;
        }],
        bindings: {
            isLoading: '&',
            whenFinished: '<',
            whenLoading: '@'
        }
    })

    .factory('sortingService', [function () {
        var service = {};
        // example content:
        // {'id': '+', 'name': '-'}
        // '+' - ascending sort order
        // '-' - descending sort order
        // ''  - not set
        /** @type {{[a: string]: string}} */
        var sortBy = {};
        /** @type {string[]} */
        var sortByOrderedKeys = [];

        /**
         * @param {string} name
         * @returns {string}
         */
        service.getSort = function (name) {
            return sortBy[name];
        };

        /**
         * @param {string} name
         * @param {boolean} [isDefault=false]
         * @param {boolean} [isInit=false]
         */
        service.setSort = function (name, isDefault, isInit) {
            isDefault = isDefault || false;
            isInit = isInit || false;

            switch (sortBy[name]) {
                case '':
                    sortBy[name] = '+';
                    break;
                case '-':
                    sortBy[name] = '+';
                    break;
                case '+':
                    sortBy[name] = '-';
                    break;
                default:
                    // init with empty if not present in sortBy
                    sortBy[name] = isDefault ? '+' : '';
            }


            // if you want to sort by multiple columns, remove this 'if' statement
            if (!isInit) {
                /** @param {string} key */
                var key;
                for (var i = 0, l = sortByOrderedKeys.length; i < l; i++) {
                    key = sortByOrderedKeys[i];
                    if (key != name) {
                        sortBy[key] = '';
                    }
                }
            }

            if (sortBy[name] != '' && sortByOrderedKeys.indexOf(name) < 0) {
                sortByOrderedKeys.push(name);
            }
        };

        /** @param {string} name */
        service.delSort = function (name) {
            delete sortBy[name];
            sortByOrderedKeys.splice(sortByOrderedKeys.indexOf(name), 1);
        };

        service.resetSorting = function () {
            for (var i = 0, l = sortByOrderedKeys.length; i < l; i++) {
                sortBy[sortByOrderedKeys[i]] = '';
            }
            sortByOrderedKeys.splice(0, sortByOrderedKeys.length);
        };

        /** @returns {{[a: string]: string}} */
        service.getSortBy = function () {
            return JSON.parse(JSON.stringify(sortBy));
        };

        /** @returns {string[]} */
        service.getSortByOrderedKeys = function () {
            return JSON.parse(JSON.stringify(sortByOrderedKeys));
        };

        /** @type {{[a: string]: string}} */
        var defaultSortIcons = {
            '': 'fas fa-sort',
            '+': 'fas fa-sort-up',
            '-': 'fas fa-sort-down'
        };

        /**
         * @param {string} name
         * @returns {string}
         */
        service.getDefaultSortIcon = function (name) {
            return defaultSortIcons[sortBy[name]];
        };

        /** @returns {string[]} */
        service.orderByKeys = function () {
            return sortByOrderedKeys
                .filter(function (filter) {
                    return sortBy[filter] != '';
                })
                .map(function (filter) {
                    return sortBy[filter] + filter;
                });
        };

        return service;
    }])
    .factory('looseAccess', ['slashDB', function (slashdb) {
        return {
            hasAccess: function(currentConfigId, fieldValue, newItem) {
                if (currentConfigId === undefined || fieldValue === undefined || newItem === undefined) {
                    return true
                }

                let currentUser = slashdb.settings.user

                if (!Array.isArray(fieldValue)){
                    fieldValue = fieldValue.split(',')
                }


                let isAdmin = currentUser === 'admin'
                let isInList = fieldValue.includes(currentUser)

                let valid = isAdmin || newItem || !newItem && isInList;
                return valid
            },
        }

    }])

    .component('sortResetBtn', {
        template: '<a href="#" class="{{$ctrl.buttonClass}}" ' +
            'ng-class="{\'disabled\': $ctrl.isDisabled()}"' +
            'ng-click="$ctrl.sort.resetSorting()">clear sort</a>',
        bindings: {
           buttonClass: '@',
           buttonClassAppend: '@',
        },
        controller: ['sortingService', function (sortingService) {
            var ctrl = this;
            ctrl.$onInit = () => {
                var initDisabled = true;
                ctrl.sort = sortingService;

                if (!ctrl.buttonClass) {
                    ctrl.buttonClass = 'btn btn-xs btn-default';
                }

                if (ctrl.buttonClassAppend) {
                    ctrl.buttonClass = ctrl.buttonClass + ' ' + ctrl.buttonClassAppend;
                }

                ctrl.isDisabled = function () {
                    return ctrl.sort.orderByKeys().length == 0;
                }
            }
        }]
    })

    .directive('sortHeader', ['sortingService', function (sortingService) {
        return {
            restrict: 'A',
            transclude: true,
            template: '<a href="">' +
                '<span ng-transclude></span> ' +
                '<i ng-class="sort.getDefaultSortIcon(key)"></i>' +
                '</a>',
            scope: {
                key: '@',
                isDefault: '<?'
            },
            link: function (scope, element, attrs) {
                scope.sort = sortingService;

                // init sorting for this key
                scope.isDefault = scope.isDefault || false;
                sortingService.setSort(scope.key, scope.isDefault, true);

                // add 'no-wrap' class to the th element so we don't get
                // an ugly table header on smaller screens
                element.addClass('no-wrap');

                // listen for a click event on the header
                element.on('click', function () {
                    sortingService.setSort(scope.key);
                    scope.$apply();
                });
            }
        };
    }])

    .filter('trust', ['$sce', function ($sce) {
        return function (text) {
            return $sce.trustAsHtml(text);
        };
    }])

    .component('errorAlert', {
        template: ' \
        <div ng-show="$ctrl.errorMsg.length > 0" class="alert alert-danger" role="alert"> \
            <span ng-bind="$ctrl.errorMsg | trust"></span> \
            <a ng-href="" class="close" ng-click="$ctrl.errorMsg = \'\'">&times;</a> \
        </div>',
        controller: ['$scope', function ($scope) {
            var ctrl = this;
            ctrl.errorMsg = '';

            $scope.$on('sdb-request-error', function (e, errorMsg) {
                if (errorMsg) {
                    ctrl.errorMsg = 'Something went wrong: ' + errorMsg;
                }
                else {
                    ctrl.errorMsg = null
                }
            });
        }]
    })

    .component('successAlert', {
        template: ' \
        <div ng-show="$ctrl.successMsg.length > 0" class="alert alert-success" role="alert"> \
            <span ng-bind-html="$ctrl.successMsg | trust"></span> \
            <a ng-href="" class="close" ng-click="$ctrl.successMsg = \'\'">&times;</a> \
        </div>',
        controller: ['$scope', function ($scope) {
            var ctrl = this;
            ctrl.successMsg = '';

            $scope.$on('sdb-request-success', function (e, successMsg) {
                ctrl.successMsg = successMsg;
            });
        }]
    })

    .component('friendlyErrors', {
        template: '<p class="alert alert-danger" ng-show="$ctrl.errorMessage" ng-bind="$ctrl.errorMessage"></p>',
        bindings: {
            configErrors: '<',
            dbType: '<'
        },
        controller: ['$scope', function($scope){
            var ctrl = this;

            var fieldMappingDbDef = {
                'autoconnect': 'Connect Automatically',
                'autoload': 'Auto Discover',
                'connection': 'Database File Location',
                'db_encoding': 'Database Charset',
                'db_id': 'Database ID',
                'db_type': 'Type',
                'db_port': 'Port',
                'db_http_path': 'HTTP Path',
                'desc': 'Description',
                'use_user_db_credentials_for_mapping': 'Use Credentials For Current User',
                'read': 'View DB Config',
                'write': 'Modify DB config',
                'execute': 'Connect / Disconnect DB'
            };

            function getUserFriendlyErrorMessages (errors) {
                var errorMsgs = errors.map(
                    function (err, _, __) {
                        switch (err['type']) {
                            case 'MISSING':
                                if (err['field'] !== 'connection') {
                                    return fieldMappingDbDef[err['field']] +
                                        ' is obligatory'
                                }

                                if (ctrl.dbType === 'sqlite') {
                                    return 'Database File Location is obligatory'
                                }

                                // connection
                                return 'Database Host, Name or ' +
                                    'Connection String is obligatory'

                            case 'EXTRA':
                                return err['description'] + " - try to reload page"
                            case null:
                                // when response has not any defined errors
                                return err['description']
                        }

                        switch (err['field']) {
                            case 'db_id':
                                if (err['description'].includes('already exists')) {
                                    return 'Database already exists'
                                }
                                break;
                            case 'connection':
                                if (ctrl.dbType === 'sqlite') {
                                    return 'Database File Location is invalid'
                                }
                                if (err['description'] === "None is not of type 'string'") {
                                    return 'Database Host, Name or ' +
                                        'Connection String is obligatory'
                                }
                                return err['description']
                            case 'execute':
                                return '"Connect / Disconnect DB": ' + err['description']
                            case 'read':
                                return '"View DB Config" ' + err['description']
                            case 'write':
                                return '"Modify DB Config": ' + err['description']
                        }

                        if (err['description'].includes('is too long')) {
                            return fieldMappingDbDef[err['field']] + ' is too long'
                        }

                        if (err['description'].includes('out of range') && err['field'] === 'db_port') {
                            return err['description']
                        }

                        // default
                        return fieldMappingDbDef[err['field']] + ' is invalid: ' + err['description']
                    }
                ).join(', ') + '.';
                return 'Errors found in database definition: ' + errorMsgs;
            }

            ctrl.$onChanges = function(changes){
                if (ctrl.configErrors.length > 0) {
                    ctrl.errorMessage = getUserFriendlyErrorMessages(ctrl.configErrors);
                } else {
                    ctrl.errorMessage = null;
                }
            };
        }]
    })

    .factory('deleteResourceService',
        ['$rootScope', '$window', '$timeout', 'slashDB', 'sessionControlService',
        function ($rootScope, $window, $timeout, slashdb, sessionControlService) {
        var service = {
            tilRedirect: 0
        };

        /**
         * @param {string} url
         * @param {string} urlPostfix
         */
        service.deleteResource = function (url, urlPostfix) {
            slashdb.delete(url)
                .then(function (resp) {
                    /** @type {string} */
                    var newUrl = $window.location.href.split("/").slice(0, 6).join("/") + '.html' + urlPostfix;
                    /** @type {string} */
                    var successMsg = 'Successfully deleted resource, redirecting to list view...' +
                        'If nothing happens in ' + service.tilRedirect + 's, please click ' +
                        '<a href="' + newUrl + '">here</a>.';

                    $rootScope.$broadcast('sdb-request-success', successMsg);
                    $timeout(function () {
                        $window.location.href = newUrl;
                    }, 1000 * service.tilRedirect);
                })
                .catch(function (resp) {
                    /** @type {string} */
                    var errorMsg;
                    let data = resp.data
                    if (data.http_code != null && data.description != null) {
                        errorMsg = data.http_code + ': ' + data.description;
                    } else {
                        errorMsg = 'Unable to delete this object.';
                    }
                    $rootScope.$broadcast('sdb-request-error', errorMsg);
                })
                .finally(() => {
                    sessionControlService.onSessionLost(() =>
                        $rootScope.$broadcast('sdb-request-error', sessionControlService.expiredMsg)
                    )

                    sessionControlService.onSessionRestore(() => {
                        // refresh user
                        slashdb.setupSettings()
                        $rootScope.$broadcast('sdb-request-error', null);
                    })
                });
        };

        return service;
    }])

    .component('deleteResourceBtn', {
        template: '<button class="btn btn-xs btn-danger" type="button" \
            ng-click="$ctrl.confirmVisible = true" ng-disabled="$ctrl.deleteBtnDisabled">\
            <i class="fas fa-trash-alt"></i> Delete</button> \
            <span style="padding-left: 15px" ng-show="$ctrl.confirmVisible"> Are you sure? \
                <a href="" style="padding-right:15px; padding-left: 10px;" \
                ng-click="$ctrl.deleteResource()">Yes</a> \
                <a href="" ng-click="$ctrl.confirmVisible = false">No</a> \
            </span>',
        controller: ['$scope', 'deleteResourceService', function ($scope, deleteResourceService) {
            var ctrl = this;
            ctrl.deleteBtnDisabled = false;
            ctrl.confirmVisible = false;
            ctrl.urlPostfix = ctrl.urlPostfix || '';
            deleteResourceService.tilRedirect = ctrl.tilRedirect || 2;

            $scope.$on('sdb-request-error', function () {
                ctrl.deleteBtnDisabled = false;
            });

            ctrl.deleteResource = function () {
                deleteResourceService.deleteResource(ctrl.url, ctrl.urlPostfix);
                ctrl.confirmVisible = false;
                ctrl.deleteBtnDisabled = true;
            };
        }],
        bindings: {
            url: '@',
            urlPostfix: '@?',
            tilRedirect: '<?'
        }
    })

    .component('editableResource', {
        transclude: true,
        template: '<div ng-transclude></div>',
        controller: ['$rootScope', '$scope', '$timeout', '$window', 'slashDB', 'sessionControlService',
        function ($rootScope, $scope, $timeout, $window, slashdb, sessionControlService) {
            var ctrl = this;
            ctrl.isEditMode = false;
            ctrl.showInputs = false;
            ctrl.initData = {};
            ctrl.data = {};
            ctrl.nullKeys = {};
            ctrl.emptyKeys = {};
            
            $scope.$on('edit-resource', function () {
                setVisibility(true);

                ctrl.emptyDisplayString = slashdb.settings['html_sdb.datadiscovery.empty_display_str'];
                ctrl.nullDisplayString = slashdb.settings['html_sdb.datadiscovery.null_display_str'];

                // when in edit mode - change null/empty fields from placeholders to
                // null/empty strings                
                for (key in ctrl.data) {

                    if (ctrl.nullKeys[key] === true) {
                        ctrl.data[key] = null;
                        ctrl.initData[key] = null;
                    }
                    if (ctrl.emptyKeys[key] === true) {
                        ctrl.data[key] = '';
                        ctrl.initData[key] = '';
                    }
                }  

            });

            $scope.$on('set-null', function (evt, key) {
                
                // when null button clicked - if field is currently null, set to empty string,
                if (ctrl.data[key] === null) {
                    ctrl.data[key] = '';
                    ctrl.nullKeys[key] = false;
                    ctrl.emptyKeys[key] = true;
                }
                else {
                    ctrl.data[key] = null;
                    ctrl.nullKeys[key] = true;
                    ctrl.emptyKeys[key] = false;
                }
            });            


            /**
             * @param {string} key
             * @param {any} value
             */
            ctrl.initModel = function (key, value) {
                ctrl.initData[key] = value;
                ctrl.data[key] = value;

                for (key in ctrl.data) {
                    const nullElement = document.querySelector(`#col-${key} div input.null-val`);
                    const emptyElement = document.querySelector(`#col-${key} div input.empty-val`);

                    if (nullElement !== null) {
                        ctrl.nullKeys[key] = true;
                    }
                    else if (emptyElement !== null) {
                        ctrl.emptyKeys[key] = true;
                    }
                    else {
                        ctrl.nullKeys[key] = false;
                        ctrl.emptyKeys[key] = false;
                    }
                }    

            };

           
            /**
             * @param {boolean} value
             * @param {boolean} [reversed=false]
             */
            var setVisibility = function (value, reversed) {
                reversed = reversed || false;
                /** @type {number} */
                var timeout = 200;

                if (reversed) {
                    ctrl.showInputs = value;
                    $timeout(function () {
                        ctrl.isEditMode = value;
                    }, timeout);
                    return;
                }

                ctrl.isEditMode = value;
                $timeout(function () {
                    ctrl.showInputs = value;
                }, timeout);
            };

            /** @returns {{[a: string]: string}} */
            var getChangedData = function () {
                /** @type {string} */
                var key;
                /** @type {string} */
                var newValue;
                var keys = Object.keys(ctrl.data);
                /** @type {{[a: string]: string}} */
                var changedData = {};

                for (var i = 0, l = keys.length; i < l; i++) {
                    key = keys[i];
                    newValue = ctrl.data[key];
                    if (ctrl.initData[key] != newValue) {
                        changedData[key] = newValue;
                    }
                }
                return changedData;
            };

            /** @returns {{redirectUrl: string, pkChanged: boolean}} */
            var getUrlPKInfo = function () {
                /** @type {string[]} */
                // ignore QS
                var urlParts = $window.location.href.split('?')[0].split('/');
                // get pk parts. 6 is index of pk parts in single resource url
                var restParts = urlParts.slice(6),
                    pkChanged = false;

                var dataKeys = Object.keys(ctrl.data);
                /**
                 *  @type {string} key
                 *  @type {string} urlValue
                 *  @type {string} dataValue
                 */
                var key, urlValue, dataValue;
                var lastIdx = restParts.length - 1;

                for (var i = restParts.length - 1; i > 0; i -= 2) {
                    key = restParts[i - 1];
                    if (dataKeys.indexOf(key) === -1) {
                        break;
                    }

                    urlValue = i == lastIdx ? rmSuffix(restParts[i]) : restParts[i];
                    restParts[i] = urlValue;
                    dataValue = ctrl.data[key];
                    if (dataValue != urlValue) {
                        pkChanged = true;
                        restParts[i] = dataValue;
                    }
                }

                /** @type {string[]} */
                var tmp = [];
                Array.prototype.push.apply(tmp, urlParts.slice(0, 6));
                Array.prototype.push.apply(tmp, restParts);
                return {
                    redirectUrl: tmp.join('/') + '.html',
                    pkChanged: pkChanged
                };
            };

            /** @returns {string} */
            var getResourceUrl = function () {
                // strip extension (4 characters!) and qs
                return ctrl.url.split('?')[0].slice(0, -5)
            };

            var endEditMode = function () {
                $rootScope.$broadcast('close-edit-resource');
                setVisibility(false, true);
            };

            /**
             * @param {string} redirectUrl
             * @param {number} [tilRedirect=2]
             * */
            var onPKChanged = function (redirectUrl, tilRedirect) {
                if (tilRedirect == null) {
                    tilRedirect = 2;
                }
                var successMsg = 'Successfully updated "' + getResourceUrl() + '". ' +
                    'One or more PK fields changed, redirecting to new location... ' +
                    'If nothing happens in ' + tilRedirect + 's, please click ' +
                    '<a href="' + redirectUrl + '">here</a>.';
                $rootScope.$broadcast('sdb-request-success', successMsg);
                endEditMode();

                $timeout(function () {
                    $window.location.href = redirectUrl;
                }, 1000 * tilRedirect);
            };

            var onStdSave = function () {
                $rootScope.$broadcast('sdb-request-success', 'Successfully updated "' + getResourceUrl() + '".');
                ctrl.setNullEmptyValuesViewMode(ctrl.data);
                // @ts-ignore
                angular.extend(ctrl.initData, ctrl.data);
                endEditMode();
            };

            ctrl.save = function () {
                var changedData = getChangedData();
                if (Object.keys(changedData).length == 0) {
                    onStdSave();
                    return;
                }

                // remove limit from QS
                let [resourceUrl, qs] = ctrl.url.split('?')
                let qsObj = new URLSearchParams(qs)

                let url
                if (qsObj.has('limit')) {
                    qsObj.delete('limit')
                    qs = qsObj.toString()
                    url = resourceUrl + '?' + qs
                } else {
                    url = ctrl.url
                }

                slashdb.put(url, changedData)
                    .then(function (resp) {
                        var urlPkInfo = getUrlPKInfo();
                        if (urlPkInfo.pkChanged) {
                            onPKChanged(urlPkInfo.redirectUrl);
                        } else {
                            onStdSave();
                        }
                    })
                    .catch(function (resp) {
                        var errorMsg = resp.data.http_code + ': ' + resp.data.description;
                        $rootScope.$broadcast('sdb-request-error', errorMsg);
                    })
                    .finally(() => {
                        sessionControlService.onSessionLost(() =>
                            $rootScope.$broadcast(
                                'sdb-request-error',
                                sessionControlService.expiredMsg
                            )
                        )
                        sessionControlService.onSessionRestore(() => {
                            // refresh user
                            slashdb.setupSettings();
                            $rootScope.$broadcast('sdb-request-error', '');

                        })
                    });
            };

            ctrl.close = function () {
                ctrl.setNullEmptyValuesViewMode(ctrl.initData);
                // @ts-ignore
                angular.extend(ctrl.data, ctrl.initData);
                endEditMode();
            };

            // updates null/empty status of input fields when exiting edit mode (save or close),
            // also updates display values to placeholders
            ctrl.setNullEmptyValuesViewMode = function(fields) {
                for (const key in fields) {

                    if (fields[key] === null) {
                        fields[key] = ctrl.nullDisplayString;
                        ctrl.emptyKeys[key] = false;
                        ctrl.nullKeys[key] = true;
                    }

                    else if (fields[key] === '') {
                        fields[key] = ctrl.emptyDisplayString;
                        ctrl.emptyKeys[key] = true;
                        ctrl.nullKeys[key] = false;

                    }

                    else {
                        ctrl.emptyKeys[key] = false;
                        ctrl.nullKeys[key] = false;
                    }
                }
            }
        }],
        bindings: {
            url: '@',
        }
    })

    .component('editResourceBtn', {
        template: '<button type="button" class="btn btn-primary btn-xs" \
                           ng-disabled="$ctrl.isEditMode" ng-click="$ctrl.startEditing()"> \
                       <i class="fas fa-edit"></i> Edit \
                   </button>',
        controller: ['$rootScope', '$scope', function ($rootScope, $scope) {
            var ctrl = this;
            ctrl.isEditMode = false;

            ctrl.startEditing = function () {
                $rootScope.$broadcast('edit-resource');
                ctrl.isEditMode = true;
            };

            $scope.$on('close-edit-resource', function () {
                ctrl.isEditMode = false;
            });
        }]
    })

    .component('nullBtn', {
        template: '<button class="btn btn-default btn-sm null" type="button" \
                           ng-disabled="$ctrl.isEditMode" ng-click="$ctrl.setValueNull()" \
                           ng-class="{\'null-active\': $ctrl.isNull == true}"> \
                       NULL \
                   </button>',
        controller: ['$rootScope', '$scope', 'slashDB', function ($rootScope, $scope, slashdb) {
            var ctrl = this;

            ctrl.$onInit = function () {
                ctrl.isNull = ctrl.value
                const nullElement = document.querySelector(`#col-${ctrl.key}.null-val`);
                if (nullElement !== null) {
                    ctrl.isNull = true;
                }
                else {
                    ctrl.isNull = false;
                }
            }

            ctrl.setValueNull = function () {
                ctrl.isNull = ctrl.isNull === false ? true : false;
                $rootScope.$broadcast('set-null', ctrl.key);
            };

        }],
        bindings: {
            key: '@',
            value: '@'
        }        
    })

    .component('editResourceControls', {
        template: '<div ng-show="$ctrl.isEditMode"> \
                       <button type="submit" class="btn btn-success btn-sm" ng-click="$ctrl.onSave()">Save</button> \
                       <button type="button" class="btn btn-default btn-sm" ng-click="$ctrl.onClose()">Close</button> \
                   </div>',
        bindings: {
            isEditMode: '<',
            onSave: '&',
            onClose: '&'
        }
    });
