angular.module('sdbgui')
    .component('welcome', {
        templateUrl: '/static/components/sdbgui-welcome/sdbgui-welcome.template.html',
        controller: ['$window', '$http', '$timeout', '$interval', 'slashDB',
            function ($window, $http, $timeout, $interval, slashdb) {
            var ctrl = this;

            function connectChinook(redirectTo) {
                return slashdb.dbOn('Chinook').then(
                    function (response) {
                        function getConnectStatuses() {
                            slashdb.get('/settings/reflect-status.json', {}, false, false).then(
                                function (response) {
                                    let data = response.data;
                                    if (data['Chinook']['model'] === 'Connected') {
                                        location.href = redirectTo;
                                    }
                                }
                            )
                        }
                        $interval(getConnectStatuses,200, 0);
                        return response
                    }
                )
            }

            this.init = function () {
                ctrl.slashdb = slashdb;
                ctrl.modalVisible = false;
                ctrl.adminCredentials = {
                    user: 'admin',
                    password1: '',
                    password2: ''
                };
                ctrl.licFile = null;
                ctrl.errorMsg = '';

                slashdb.setupSettings()
                    .then(function (response) {
                        var settings = response.data[0];
                        // hide modal when there is no welcome mode
                        if (!settings['welcome_mode']) {
                            $window.location.reload()
                        }
                        ctrl.termsRequired = settings.require_details_in_terms_and_conditions;
                        ctrl.steps = settings.welcome_steps;
                        ctrl.modalVisible = true;
                        ctrl.step = ctrl.steps[0];
                        ctrl.amiSearchUrl = "https://console.aws.amazon.com/ec2/v2/home?#Instances:imageId=" + slashdb.settings.ami_id + ";sort=instanceId";
                    });
            };

            this.goNextStep = function () {
                ctrl.step = ctrl.steps[ctrl.steps.indexOf(this.step) + 1];
            };

            this.goPrevStep = function () {
                ctrl.step = ctrl.steps[ctrl.steps.indexOf(this.step) - 1];
            };

            this.chooseLicenseStep = function () {
                ctrl.formCtrl.selectedLicenseType.$setDirty();
                var stepValid = ctrl.formCtrl.selectedLicenseType.$valid;

                if (ctrl.selectedLicenseType == 'Production License') {
                    ctrl.formCtrl.licFile.$setDirty();
                    stepValid = stepValid && ctrl.formCtrl.licFile.$valid;
                }

                if (stepValid) ctrl.goNextStep();
            };

            this.termsAndConditionsStep = function () {
                ctrl.formCtrl.firstname.$setTouched();
                ctrl.formCtrl.lastname.$setTouched();
                ctrl.formCtrl.email.$setTouched();
                ctrl.formCtrl.terms.$setTouched();
                var stepValid = ctrl.formCtrl.firstname.$valid && ctrl.formCtrl.lastname.$valid && ctrl.formCtrl.email.$valid && ctrl.formCtrl.terms.$valid;
                if (stepValid) {
                    ctrl.goNextStep();
                }
            };

            this.adminPasswordStep = function () {
                ctrl.formCtrl.password1.$setTouched();
                ctrl.formCtrl.password2.$setTouched();
                var stepValid = ctrl.formCtrl.password1.$valid && ctrl.formCtrl.password2.$valid;
                if (slashdb.settings.ami_id){
                    ctrl.formCtrl.instanceId.$setTouched();
                    stepValid = stepValid && ctrl.formCtrl.instanceId.$valid;
                }

                if (stepValid) {
                    slashdb.post('/set-admin-password', ctrl.adminCredentials).then(
                        (response) => {
                            ctrl.errorMsg = '';
                            ctrl.resetPswdValidity();
                            ctrl.goNextStep();
                        }
                    ).catch(
                        (response) => {
                            ctrl.errorMsg = response.data.description;
                            ctrl.formCtrl.password1.$setValidity('server', false);
                            ctrl.formCtrl.password2.$setValidity('server', false);
                        }
                    )
                }
            };

            this.resetPswdValidity = () => {
                ctrl.formCtrl.password1.$setValidity('server', true);
                ctrl.formCtrl.password2.$setValidity('server', true);
            }

            this.register = function (redirectTo) {
                var stepValid = ctrl.formCtrl.password1.$valid && ctrl.formCtrl.password2.$valid;
                if (!stepValid) {
                    return
                }

                slashdb.login(ctrl.adminCredentials.user, ctrl.adminCredentials.password1)
                    .then(() => {
                        switch(ctrl.selectedLicenseType) {
                            case 'Developer License':
                                return ctrl._installDevLicense();
                            case 'Production License':
                                return slashdb.uploadLicense(ctrl.licFile);
                            default:
                                return slashdb.get('/license.json', {}, false, false);
                        }
                }).then(function (response) {
                    var sdb_lic = response.data,
                        dbtypes = sdb_lic.dbtypes,

                        data = {
                            created: new Date().toISOString().replace('Z', ''),
                            first_name: ctrl.firstname,
                            last_name: ctrl.lastname,
                            email: ctrl.email,
                            product: sdb_lic.product,
                            version: sdb_lic.version,
                            expiration: sdb_lic.expiration,
                            mssql: dbtypes.indexOf('mssql') >= 0 ? 1 : 0,
                            mysql: dbtypes.indexOf('mysql') >= 0 ? 1 : 0,
                            sqlite: dbtypes.indexOf('sqlite') >= 0 ? 1 : 0,
                            oracle: dbtypes.indexOf('oracle') >= 0 ? 1 : 0,
                            postgresql: dbtypes.indexOf('postgresql') >= 0 ? 1 : 0,
                            max_dbs: sdb_lic['max-databases'],
                            max_users: sdb_lic['max-users'],
                            hostname: sdb_lic['hostname-fqdn'],
                            org: sdb_lic.organization,
                            org_dept: sdb_lic['organization-dept'],
                            bill_name: sdb_lic['billing-contact-name'],
                            bill_email: sdb_lic['billing-contact-email'],
                            bill_phone: sdb_lic['billing-contact-phone'],
                            tech_name: sdb_lic['technical-contact-name'],
                            tech_email: sdb_lic['technical-contact-email'],
                            tech_phone: sdb_lic['technical-contact-phone'],
                            key: sdb_lic.key,
                            license_type: sdb_lic['license-type']
                        };

                    if (slashdb.settings.registration_url) {
                        return $http.post(slashdb.settings.registration_url, data, {timeout: 10*1000});
                    }
                    return response
                })
                .finally(function(response){
                    return slashdb.get('/finish-initialization', {}, false, false);
                })
                .finally(function (response) {
                    if (redirectTo === '/db/Chinook') {
                        return connectChinook(redirectTo);
                    } else {
                        location.href = redirectTo;
                    }
                })
            };

            this._installDevLicense = function () {
                var name;

                if (ctrl.firstname || ctrl.lastname) {
                    name = ctrl.firstname + ' ' + ctrl.lastname
                } else {
                    name = 'Unregistered'
                }

                var data = {
                    'organization': name,
                    'billing-contact-name': name,
                    'billing-contact-email': ctrl.email || ''
                };
                return slashdb.post('/install-dev-license', data);
            };

            this.submit = function () {
                ctrl.alert = '';

                switch (ctrl.step) {
                    case 'choose-license':
                        ctrl.chooseLicenseStep();
                        break;
                    case 'terms-and-conditions':
                        ctrl.termsAndConditionsStep();
                        break;
                    case 'admin-password':
                        ctrl.adminPasswordStep();
                        break;
                    case 'done':
                        break;
                }
            };

            this.init();
        }]
    });