angular.module 'sdbgui'
.factory 'QueryExecService', ['$rootScope', '$location', 'slashDB', ($rootScope, $location, slashdb) ->
  new class QueryExecService
    constructor: ->
      @queryBasePath = $location.protocol() + '://' + $location.host()
      if $location.port and $location.port != 80
        @queryBasePath = @queryBasePath + ':' + $location.port()
      @queryBasePath = @queryBasePath + '/query'
      @loadMoreVisible = false
      @nullPlaceHolder = '<SDB_NULL>'

      @resetQueryOutput()

    resetQueryOutput: ->
      @queryOutput =
        columnNames: []
        data: []

    getExecPath: (queryId, queryParameters, limit, offset=0, count, format,
      distinct=false, transpose=false, stream=false, nullStr=false, order_by=null) ->
      url = '/' + queryId

      qs_url_substitution = {}

      for paramName, paramValue of queryParameters
        if paramValue
          for key, value of slashdb.settings.reversed_url_substitution
            if key == '.' and (paramValue != '.' and paramValue != '..')
              # skip dots, because they special
              continue
            else if key in paramValue
                paramValue = paramValue.replace(new RegExp(key,'g'), value)
                qs_url_substitution[key] = value
        else
          paramValue = ''

        encodedParamValue = encodeURIComponent(paramValue)
        url = "#{url}/#{paramName}/#{encodedParamValue}"

      qs = []
      if limit? && limit > -1
        qs.push "limit=#{limit}"
      if offset > 0
        qs.push "offset=#{offset}"
      if count
        qs.push "count"
      if distinct
        qs.push "distinct"
      if transpose
        qs.push "transpose"
      if stream
        qs.push "stream=true"
      if nullStr
        qs.push "nullStr=#{@nullPlaceHolder}"
      if order_by
        qs.push "sort=#{order_by}"
      if qs_url_substitution
        for key, value of qs_url_substitution
            qs.push "#{value}=#{key}"
      qs = qs.join('&')
      if qs
        qs = "?#{qs}"

      url = url + '.' + format + qs
      return url

    executeQuery: (queryId, httpMethod, data, params, headers) ->
      @resetQueryOutput()

      httpSettings =
        params: params
        headers: headers

      if httpMethod in ['GET', 'DELETE']
        # when making GET requests query parameters are passed in url
        url = @getExecPath queryId, data, null, null, null, 'json'
        data = null
      else
        url = '/' + queryId + '.json'
      return slashdb.executeQuery url, httpMethod, data, httpSettings
      .then (response) =>
        if response.data.length > 0
          @queryOutput.columnNames = JSON.parse response.headers 'columns'
          @queryOutput.numericColumns = JSON.parse response.headers 'numeric_columns'
          @queryOutput.data = response.data
          @loadMoreVisible = true
        $rootScope.$broadcast 'queryExecuted', {httpResponse: response}
        return
      .catch (response) =>
        @queryOutput.columnNames = []
        @queryOutput.data = []
        $rootScope.$broadcast 'queryExecuted', {httpResponse: response}
        return
]